@@ -42,8 +42,8 @@ static std::map<RedisType, std::string> type_to_cmd = {
     {kRedisZSet, "zadd"},  {kRedisBitmap, "setbit"}, {kRedisSortedint, "siadd"}, {kRedisStream, "xadd"},
 };
 
-SlotMigrator::SlotMigrator(Server *svr, int max_migration_speed, int max_pipeline_size, int seq_gap_limit)
-    : Database(svr->storage, kDefaultNamespace), svr_(svr) {
+SlotMigrator::SlotMigrator(Server *srv, int max_migration_speed, int max_pipeline_size, int seq_gap_limit)
+    : Database(srv->storage, kDefaultNamespace), srv_(srv) {
   // Let metadata_cf_handle_ be nullptr, and get them in real time to avoid accessing invalid pointer,
   // because metadata_cf_handle_ and db_ will be destroyed if DB is reopened.
   // [Situation]:
@@ -71,7 +71,7 @@ SlotMigrator::SlotMigrator(Server *svr, int max_migration_speed, int max_pipelin
     seq_gap_limit_ = seq_gap_limit;
   }
 
-  if (svr->IsSlave()) {
+  if (srv->IsSlave()) {
     SetStopMigrationFlag(true);
   }
 }
@@ -92,9 +92,9 @@ Status SlotMigrator::PerformSlotMigration(const std::string &node_id, std::strin
 
   migration_state_ = MigrationState::kStarted;
 
-  auto speed = svr_->GetConfig()->migrate_speed;
-  auto seq_gap = svr_->GetConfig()->sequence_gap;
-  auto pipeline_size = svr_->GetConfig()->pipeline_size;
+  auto speed = srv_->GetConfig()->migrate_speed;
+  auto seq_gap = srv_->GetConfig()->sequence_gap;
+  auto pipeline_size = srv_->GetConfig()->pipeline_size;
 
   if (speed <= 0) {
     speed = 0;
@@ -279,7 +279,7 @@ Status SlotMigrator::startMigration() {
   dst_fd_.Reset(*result);
 
   // Auth first
-  std::string pass = svr_->GetConfig()->requirepass;
+  std::string pass = srv_->GetConfig()->requirepass;
   if (!pass.empty()) {
     auto s = authOnDstNode(*dst_fd_, pass);
     if (!s.IsOK()) {
@@ -396,7 +396,7 @@ Status SlotMigrator::finishSuccessfulMigration() {
   }
 
   std::string dst_ip_port = dst_ip_ + ":" + std::to_string(dst_port_);
-  s = svr_->cluster->SetSlotMigrated(migrating_slot_, dst_ip_port);
+  s = srv_->cluster->SetSlotMigrated(migrating_slot_, dst_ip_port);
   if (!s.IsOK()) {
     return s.Prefixed(fmt::format("failed to set slot {} as migrated to {}", migrating_slot_.load(), dst_ip_port));
   }
@@ -474,8 +474,8 @@ Status SlotMigrator::checkSingleResponse(int sock_fd) { return checkMultipleResp
 
 // Commands  |  Response            |  Instance
 // ++++++++++++++++++++++++++++++++++++++++
-// set          Redis::Integer         :1/r/n
-// hset         Redis::SimpleString    +OK/r/n
+// set          Redis::Integer         :1\r\n
+// hset         Redis::SimpleString    +OK\r\n
 // sadd         Redis::Integer
 // zadd         Redis::Integer
 // siadd        Redis::Integer
@@ -497,6 +497,7 @@ Status SlotMigrator::checkSingleResponse(int sock_fd) { return checkMultipleResp
 // sirem        Redis::Integer
 // del          Redis::Integer
 // xadd         Redis::BulkString
+// bitfield     Redis::Array           *1\r\n:0
 Status SlotMigrator::checkMultipleResponses(int sock_fd, int total) {
   if (sock_fd < 0 || total <= 0) {
     return {Status::NotOK, fmt::format("invalid arguments: sock_fd={}, count={}", sock_fd, total)};
@@ -509,7 +510,7 @@ Status SlotMigrator::checkMultipleResponses(int sock_fd, int total) {
   setsockopt(sock_fd, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));
 
   // Start checking response
-  size_t bulk_len = 0;
+  size_t bulk_or_array_len = 0;
   int cnt = 0;
   parser_state_ = ParserState::ArrayLen;
   UniqueEvbuf evbuf;
@@ -534,14 +535,20 @@ Status SlotMigrator::checkMultipleResponses(int sock_fd, int total) {
 
           if (line[0] == '-') {
             return {Status::NotOK, fmt::format("got invalid response of length {}: {}", line.length, line.get())};
-          } else if (line[0] == '$') {
+          } else if (line[0] == '$' || line[0] == '*') {
             auto parse_result = ParseInt<uint64_t>(std::string(line.get() + 1, line.length - 1), 10);
             if (!parse_result) {
               return {Status::NotOK, "protocol error: expected integer value"};
             }
 
-            bulk_len = *parse_result;
-            parser_state_ = bulk_len > 0 ? ParserState::BulkData : ParserState::OneRspEnd;
+            bulk_or_array_len = *parse_result;
+            if (bulk_or_array_len <= 0) {
+              parser_state_ = ParserState::OneRspEnd;
+            } else if (line[0] == '$') {
+              parser_state_ = ParserState::BulkData;
+            } else {
+              parser_state_ = ParserState::ArrayData;
+            }
           } else if (line[0] == '+' || line[0] == ':') {
             parser_state_ = ParserState::OneRspEnd;
           } else {
@@ -552,17 +559,33 @@ Status SlotMigrator::checkMultipleResponses(int sock_fd, int total) {
         }
         // Handle bulk string response
         case ParserState::BulkData: {
-          if (evbuffer_get_length(evbuf.get()) < bulk_len + 2) {
+          if (evbuffer_get_length(evbuf.get()) < bulk_or_array_len + 2) {
             LOG(INFO) << "[migrate] Bulk data in event buffer is not complete, read socket again";
             run = false;
             break;
           }
           // TODO(chrisZMF): Check tail '\r\n'
-          evbuffer_drain(evbuf.get(), bulk_len + 2);
-          bulk_len = 0;
+          evbuffer_drain(evbuf.get(), bulk_or_array_len + 2);
+          bulk_or_array_len = 0;
           parser_state_ = ParserState::OneRspEnd;
           break;
         }
+        case ParserState::ArrayData: {
+          while (run && bulk_or_array_len > 0) {
+            evbuffer_ptr ptr = evbuffer_search_eol(evbuf.get(), nullptr, nullptr, EVBUFFER_EOL_CRLF_STRICT);
+            if (ptr.pos < 0) {
+              LOG(INFO) << "[migrate] Array data in event buffer is not complete, read socket again";
+              run = false;
+              break;
+            }
+            evbuffer_drain(evbuf.get(), ptr.pos + 2);
+            --bulk_or_array_len;
+          }
+          if (run) {
+            parser_state_ = ParserState::OneRspEnd;
+          }
+          break;
+        }
         case ParserState::OneRspEnd: {
           cnt++;
           if (cnt >= total) {
@@ -588,7 +611,7 @@ StatusOr<KeyMigrationResult> SlotMigrator::migrateOneKey(const rocksdb::Slice &k
     return {Status::NotOK, s.ToString()};
   }
 
-  if (metadata.Type() != kRedisString && metadata.Type() != kRedisStream && metadata.size == 0) {
+  if (!metadata.IsEmptyableType() && metadata.size == 0) {
     return KeyMigrationResult::kUnderlyingStructEmpty;
   }
 
@@ -896,7 +919,7 @@ void SlotMigrator::setForbiddenSlot(int16_t slot) {
   // Block server to set forbidden slot
   uint64_t during = util::GetTimeStampUS();
   {
-    auto exclusivity = svr_->WorkExclusivityGuard();
+    auto exclusivity = srv_->WorkExclusivityGuard();
     forbidden_slot_ = slot;
   }
   during = util::GetTimeStampUS() - during;
