@@ -33,7 +33,7 @@ class CommandPush : public Commander {
  public:
   CommandPush(bool create_if_missing, bool left) : left_(left), create_if_missing_(create_if_missing) {}
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
     std::vector<Slice> elems;
     for (size_t i = 2; i < args_.size(); i++) {
       elems.emplace_back(args_[i]);
@@ -41,7 +41,7 @@ class CommandPush : public Commander {
 
     uint64_t ret = 0;
     rocksdb::Status s;
-    redis::List list_db(svr->storage, conn->GetNamespace());
+    redis::List list_db(srv->storage, conn->GetNamespace());
     if (create_if_missing_) {
       s = list_db.Push(args_[1], elems, left_, &ret);
     } else {
@@ -51,7 +51,7 @@ class CommandPush : public Commander {
       return {Status::RedisExecErr, s.ToString()};
     }
 
-    svr->WakeupBlockingConns(args_[1], elems.size());
+    srv->WakeupBlockingConns(args_[1], elems.size());
 
     *output = redis::Integer(ret);
     return Status::OK();
@@ -109,8 +109,8 @@ class CommandPop : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::List list_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::List list_db(srv->storage, conn->GetNamespace());
     if (with_count_) {
       std::vector<std::string> elems;
       auto s = list_db.PopMulti(args_[1], left_, count_, &elems);
@@ -192,8 +192,8 @@ class CommandLMPop : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::List list_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::List list_db(srv->storage, conn->GetNamespace());
 
     std::vector<std::string> elems;
     std::string chosen_key;
@@ -259,8 +259,8 @@ class CommandBPop : public BlockingCommander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    svr_ = svr;
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    srv_ = srv;
     InitConnection(conn);
 
     auto s = TryPopFromList();
@@ -273,18 +273,18 @@ class CommandBPop : public BlockingCommander {
 
   void BlockKeys() override {
     for (const auto &key : keys_) {
-      svr_->BlockOnKey(key, conn_);
+      srv_->BlockOnKey(key, conn_);
     }
   }
 
   void UnblockKeys() override {
     for (const auto &key : keys_) {
-      svr_->UnblockOnKey(key, conn_);
+      srv_->UnblockOnKey(key, conn_);
     }
   }
 
   rocksdb::Status TryPopFromList() {
-    redis::List list_db(svr_->storage, conn_->GetNamespace());
+    redis::List list_db(srv_->storage, conn_->GetNamespace());
     std::string elem;
     const std::string *last_key_ptr = nullptr;
     rocksdb::Status s;
@@ -321,7 +321,7 @@ class CommandBPop : public BlockingCommander {
   bool left_ = false;
   int64_t timeout_ = 0;  // microseconds
   std::vector<std::string> keys_;
-  Server *svr_ = nullptr;
+  Server *srv_ = nullptr;
 };
 
 class CommandBLPop : public CommandBPop {
@@ -334,6 +334,129 @@ class CommandBRPop : public CommandBPop {
   CommandBRPop() : CommandBPop(false) {}
 };
 
+class CommandBLMPop : public BlockingCommander {
+ public:
+  CommandBLMPop() = default;
+  CommandBLMPop(const CommandBLMPop &) = delete;
+  CommandBLMPop &operator=(const CommandBLMPop &) = delete;
+
+  ~CommandBLMPop() override = default;
+
+  // format: BLMPOP timeout #numkeys key0 [key1 ...] <LEFT | RIGHT> [COUNT count]
+  Status Parse(const std::vector<std::string> &args) override {
+    CommandParser parser(args, 1);
+
+    auto timeout = GET_OR_RET(parser.TakeFloat());
+    timeout_ = static_cast<int64_t>(timeout * 1000 * 1000);
+
+    auto num_keys = GET_OR_RET(parser.TakeInt<uint32_t>());
+    keys_.clear();
+    keys_.reserve(num_keys);
+    for (uint32_t i = 0; i < num_keys; ++i) {
+      keys_.emplace_back(GET_OR_RET(parser.TakeStr()));
+    }
+
+    auto left_or_right = util::ToLower(GET_OR_RET(parser.TakeStr()));
+    if (left_or_right == "left") {
+      left_ = true;
+    } else if (left_or_right == "right") {
+      left_ = false;
+    } else {
+      return {Status::RedisParseErr, errInvalidSyntax};
+    }
+
+    while (parser.Good()) {
+      if (parser.EatEqICase("count") && count_ == static_cast<uint32_t>(-1)) {
+        count_ = GET_OR_RET(parser.TakeInt<uint32_t>());
+      } else {
+        return parser.InvalidSyntax();
+      }
+    }
+    if (count_ == static_cast<uint32_t>(-1)) {
+      count_ = 1;
+    }
+
+    return Status::OK();
+  }
+
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    srv_ = srv;
+    InitConnection(conn);
+
+    auto s = ExecuteUnblocked();
+    if (s.ok() || !s.IsNotFound()) {
+      return Status::OK();  // error has already been output
+    }
+
+    return StartBlocking(timeout_, output);
+  }
+
+  rocksdb::Status ExecuteUnblocked() {
+    redis::List list_db(srv_->storage, conn_->GetNamespace());
+    std::vector<std::string> elems;
+    std::string chosen_key;
+    rocksdb::Status s;
+    for (const auto &key : keys_) {
+      s = list_db.PopMulti(key, left_, count_, &elems);
+      if (s.ok() && !elems.empty()) {
+        chosen_key = key;
+        break;
+      }
+      if (!s.IsNotFound()) {
+        break;
+      }
+    }
+
+    if (s.ok()) {
+      if (!elems.empty()) {
+        conn_->GetServer()->UpdateWatchedKeysManually({chosen_key});
+        std::string elems_bulk = redis::MultiBulkString(elems);
+        conn_->Reply(redis::Array({redis::BulkString(chosen_key), std::move(elems_bulk)}));
+      }
+    } else if (!s.IsNotFound()) {
+      conn_->Reply(redis::Error("ERR " + s.ToString()));
+    }
+
+    return s;
+  }
+
+  void BlockKeys() override {
+    for (const auto &key : keys_) {
+      srv_->BlockOnKey(key, conn_);
+    }
+  }
+
+  void UnblockKeys() override {
+    for (const auto &key : keys_) {
+      srv_->UnblockOnKey(key, conn_);
+    }
+  }
+
+  bool OnBlockingWrite() override {
+    auto s = ExecuteUnblocked();
+    return !s.IsNotFound();
+  }
+
+  std::string NoopReply() override { return redis::NilString(); }
+
+  static const inline CommandKeyRangeGen keyRangeGen = [](const std::vector<std::string> &args) {
+    CommandKeyRange range;
+    range.first_key = 3;
+    range.key_step = 1;
+    // This parsing would always succeed as this cmd has been parsed before.
+    auto num_key = *ParseInt<int32_t>(args[2], 10);
+    range.last_key = range.first_key + num_key - 1;
+    return range;
+  };
+
+ private:
+  bool left_;
+  uint32_t count_ = -1;
+  int64_t timeout_ = 0;  // microseconds
+  std::vector<std::string> keys_;
+  Server *srv_ = nullptr;
+};
+
 class CommandLRem : public Commander {
  public:
   Status Parse(const std::vector<std::string> &args) override {
@@ -346,9 +469,9 @@ class CommandLRem : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
     uint64_t ret = 0;
-    redis::List list_db(svr->storage, conn->GetNamespace());
+    redis::List list_db(srv->storage, conn->GetNamespace());
     auto s = list_db.Rem(args_[1], count_, args_[3], &ret);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -375,9 +498,9 @@ class CommandLInsert : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
     int ret = 0;
-    redis::List list_db(svr->storage, conn->GetNamespace());
+    redis::List list_db(srv->storage, conn->GetNamespace());
     auto s = list_db.Insert(args_[1], args_[3], args_[4], before_, &ret);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -405,8 +528,8 @@ class CommandLRange : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::List list_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::List list_db(srv->storage, conn->GetNamespace());
     std::vector<std::string> elems;
     auto s = list_db.Range(args_[1], start_, stop_, &elems);
     if (!s.ok() && !s.IsNotFound()) {
@@ -423,8 +546,8 @@ class CommandLRange : public Commander {
 
 class CommandLLen : public Commander {
  public:
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::List list_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::List list_db(srv->storage, conn->GetNamespace());
     uint64_t count = 0;
     auto s = list_db.Size(args_[1], &count);
     if (!s.ok() && !s.IsNotFound()) {
@@ -448,8 +571,8 @@ class CommandLIndex : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::List list_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::List list_db(srv->storage, conn->GetNamespace());
     std::string elem;
     auto s = list_db.Index(args_[1], index_, &elem);
     if (!s.ok() && !s.IsNotFound()) {
@@ -480,8 +603,8 @@ class CommandLSet : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::List list_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::List list_db(srv->storage, conn->GetNamespace());
     auto s = list_db.Set(args_[1], index_, args_[3]);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -514,8 +637,8 @@ class CommandLTrim : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::List list_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::List list_db(srv->storage, conn->GetNamespace());
     auto s = list_db.Trim(args_[1], start_, stop_);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -532,8 +655,8 @@ class CommandLTrim : public Commander {
 
 class CommandRPopLPUSH : public Commander {
  public:
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::List list_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::List list_db(srv->storage, conn->GetNamespace());
     std::string elem;
     auto s = list_db.LMove(args_[1], args_[2], /*src_left=*/false, /*dst_left=*/true, &elem);
     if (!s.ok() && !s.IsNotFound()) {
@@ -563,8 +686,8 @@ class CommandLMove : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::List list_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::List list_db(srv->storage, conn->GetNamespace());
     std::string elem;
     auto s = list_db.LMove(args_[1], args_[2], src_left_, dst_left_, &elem);
     if (!s.ok() && !s.IsNotFound()) {
@@ -607,11 +730,11 @@ class CommandBLMove : public BlockingCommander {
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    svr_ = svr;
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    srv_ = srv;
     InitConnection(conn);
 
-    redis::List list_db(svr->storage, conn->GetNamespace());
+    redis::List list_db(srv->storage, conn->GetNamespace());
     std::string elem;
     auto s = list_db.LMove(args_[1], args_[2], src_left_, dst_left_, &elem);
     if (!s.ok() && !s.IsNotFound()) {
@@ -625,12 +748,12 @@ class CommandBLMove : public BlockingCommander {
     return StartBlocking(timeout_, output);
   }
 
-  void BlockKeys() override { svr_->BlockOnKey(args_[1], conn_); }
+  void BlockKeys() override { srv_->BlockOnKey(args_[1], conn_); }
 
-  void UnblockKeys() override { svr_->UnblockOnKey(args_[1], conn_); }
+  void UnblockKeys() override { srv_->UnblockOnKey(args_[1], conn_); }
 
   bool OnBlockingWrite() override {
-    redis::List list_db(svr_->storage, conn_->GetNamespace());
+    redis::List list_db(srv_->storage, conn_->GetNamespace());
     std::string elem;
     auto s = list_db.LMove(args_[1], args_[2], src_left_, dst_left_, &elem);
     if (!s.ok() && !s.IsNotFound()) {
@@ -652,7 +775,7 @@ class CommandBLMove : public BlockingCommander {
   bool src_left_;
   bool dst_left_;
   int64_t timeout_ = 0;  // microseconds
-  Server *svr_ = nullptr;
+  Server *srv_ = nullptr;
 };
 
 class CommandLPos : public Commander {
@@ -692,8 +815,8 @@ class CommandLPos : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::List list_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::List list_db(srv->storage, conn->GetNamespace());
     std::vector<int64_t> indexes;
     auto s = list_db.Pos(args_[1], args_[2], spec_, &indexes);
     if (!s.ok() && !s.IsNotFound()) {
@@ -727,6 +850,7 @@ class CommandLPos : public Commander {
 
 REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandBLPop>("blpop", -3, "write no-script", 1, -2, 1),
                         MakeCmdAttr<CommandBRPop>("brpop", -3, "write no-script", 1, -2, 1),
+                        MakeCmdAttr<CommandBLMPop>("blmpop", -5, "write no-script", CommandBLMPop::keyRangeGen),
                         MakeCmdAttr<CommandLIndex>("lindex", 3, "read-only", 1, 1, 1),
                         MakeCmdAttr<CommandLInsert>("linsert", 5, "write", 1, 1, 1),
                         MakeCmdAttr<CommandLLen>("llen", 2, "read-only", 1, 1, 1),
