@@ -26,6 +26,7 @@
 #include "error_constants.h"
 #include "event_util.h"
 #include "server/server.h"
+#include "time_util.h"
 #include "types/redis_stream.h"
 
 namespace redis {
@@ -131,7 +132,7 @@ class CommandXAdd : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::StreamAddOptions options;
     options.nomkstream = nomkstream_;
     if (with_max_len_) {
@@ -144,7 +145,7 @@ class CommandXAdd : public Commander {
     }
     options.next_id_strategy = std::move(next_id_strategy_);
 
-    redis::Stream stream_db(svr->storage, conn->GetNamespace());
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
     StreamEntryID entry_id;
     auto s = stream_db.Add(stream_name_, options, name_value_pairs_, &entry_id);
     if (!s.ok() && !s.IsNotFound()) {
@@ -158,7 +159,7 @@ class CommandXAdd : public Commander {
 
     *output = redis::BulkString(entry_id.ToString());
 
-    svr->OnEntryAddedToStream(conn->GetNamespace(), stream_name_, entry_id);
+    srv->OnEntryAddedToStream(conn->GetNamespace(), stream_name_, entry_id);
 
     return Status::OK();
   }
@@ -189,8 +190,8 @@ class CommandXDel : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::Stream stream_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
     uint64_t deleted = 0;
     auto s = stream_db.DeleteEntries(args_[1], ids_, &deleted);
     if (!s.ok()) {
@@ -249,11 +250,44 @@ class CommandXGroup : public Commander {
       return Status::OK();
     }
 
+    if (subcommand_ == "createconsumer") {
+      if (args.size() != 5) {
+        return {Status::RedisParseErr, errWrongNumOfArguments};
+      }
+      consumer_name_ = GET_OR_RET(parser.TakeStr());
+
+      return Status::OK();
+    }
+
+    if (subcommand_ == "setid") {
+      if (args.size() != 5 && args.size() != 7) {
+        return {Status::RedisParseErr, errWrongNumOfArguments};
+      }
+
+      xgroup_create_options_.last_id = GET_OR_RET(parser.TakeStr());
+      if (args.size() == 7) {
+        if (parser.EatEqICase("entriesread")) {
+          auto parse_result = parser.TakeInt<int64_t>();
+          if (!parse_result.IsOK()) {
+            return {Status::RedisParseErr, errValueNotInteger};
+          }
+          if (parse_result.GetValue() < 0 && parse_result.GetValue() != -1) {
+            return {Status::RedisParseErr, "value for ENTRIESREAD must be positive or -1"};
+          }
+          xgroup_create_options_.entries_read = parse_result.GetValue();
+        } else {
+          return parser.InvalidSyntax();
+        }
+      }
+
+      return Status::OK();
+    }
+
     return {Status::RedisParseErr, "unknown subcommand"};
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::Stream stream_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
 
     if (subcommand_ == "create") {
       auto s = stream_db.CreateGroup(stream_name_, xgroup_create_options_, group_name_);
@@ -278,6 +312,25 @@ class CommandXGroup : public Commander {
       }
     }
 
+    if (subcommand_ == "createconsumer") {
+      int created_number = 0;
+      auto s = stream_db.CreateConsumer(stream_name_, group_name_, consumer_name_, &created_number);
+      if (!s.ok()) {
+        return {Status::RedisExecErr, s.ToString()};
+      }
+
+      *output = redis::Integer(created_number);
+    }
+
+    if (subcommand_ == "setid") {
+      auto s = stream_db.GroupSetId(stream_name_, group_name_, xgroup_create_options_);
+      if (!s.ok()) {
+        return {Status::RedisExecErr, s.ToString()};
+      }
+
+      *output = redis::SimpleString("OK");
+    }
+
     return Status::OK();
   }
 
@@ -285,6 +338,7 @@ class CommandXGroup : public Commander {
   std::string subcommand_;
   std::string stream_name_;
   std::string group_name_;
+  std::string consumer_name_;
   StreamXGroupCreateOptions xgroup_create_options_;
 };
 
@@ -311,8 +365,8 @@ class CommandXLen : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::Stream stream_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
     uint64_t len = 0;
     auto s = stream_db.Len(args_[1], options_, &len);
     if (!s.ok()) {
@@ -333,7 +387,7 @@ class CommandXInfo : public Commander {
   Status Parse(const std::vector<std::string> &args) override {
     auto val = util::ToLower(args[1]);
     if (val == "stream" && args.size() >= 2) {
-      stream_ = true;
+      subcommand_ = "stream";
 
       if (args.size() > 3 && util::ToLower(args[3]) == "full") {
         full_ = true;
@@ -347,24 +401,39 @@ class CommandXInfo : public Commander {
 
         count_ = *parse_result;
       }
+    } else if (val == "groups" && args.size() == 3) {
+      subcommand_ = "groups";
+    } else if (val == "consumers" && args.size() == 4) {
+      subcommand_ = "consumers";
+    } else {
+      return {Status::RedisParseErr, errUnknownSubcommandOrWrongArguments};
     }
+
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    if (stream_) {
-      return getStreamInfo(svr, conn, output);
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    if (subcommand_ == "stream") {
+      return getStreamInfo(srv, conn, output);
+    }
+
+    if (subcommand_ == "groups") {
+      return getGroupInfo(srv, conn, output);
+    }
+
+    if (subcommand_ == "consumers") {
+      return getConsumerInfo(srv, conn, output);
     }
     return Status::OK();
   }
 
  private:
   uint64_t count_ = 10;  // default Redis value
-  bool stream_ = false;
+  std::string subcommand_;
   bool full_ = false;
 
-  Status getStreamInfo(Server *svr, Connection *conn, std::string *output) {
-    redis::Stream stream_db(svr->storage, conn->GetNamespace());
+  Status getStreamInfo(Server *srv, Connection *conn, std::string *output) {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
     redis::StreamInfo info;
     auto s = stream_db.GetStreamInfo(args_[2], full_, count_, &info);
     if (!s.ok() && !s.IsNotFound()) {
@@ -419,6 +488,76 @@ class CommandXInfo : public Commander {
 
     return Status::OK();
   }
+
+  Status getGroupInfo(Server *srv, Connection *conn, std::string *output) {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
+    std::vector<std::pair<std::string, StreamConsumerGroupMetadata>> result_vector;
+    auto s = stream_db.GetGroupInfo(args_[2], result_vector);
+    if (!s.ok() && !s.IsNotFound()) {
+      return {Status::RedisExecErr, s.ToString()};
+    }
+
+    if (s.IsNotFound()) {
+      return {Status::RedisExecErr, errNoSuchKey};
+    }
+
+    output->append(redis::MultiLen(result_vector.size()));
+    for (auto const &it : result_vector) {
+      output->append(redis::MultiLen(12));
+      output->append(redis::BulkString("name"));
+      output->append(redis::BulkString(it.first));
+      output->append(redis::BulkString("consumers"));
+      output->append(redis::Integer(it.second.consumer_number));
+      output->append(redis::BulkString("pending"));
+      output->append(redis::Integer(it.second.pending_number));
+      output->append(redis::BulkString("last-delivered-id"));
+      output->append(redis::BulkString(it.second.last_delivered_id.ToString()));
+      output->append(redis::BulkString("entries-read"));
+      if (it.second.entries_read == -1) {
+        output->append(redis::NilString());
+      } else {
+        output->append(redis::Integer(it.second.entries_read));
+      }
+      output->append(redis::BulkString("lag"));
+      if (it.second.lag == UINT64_MAX) {
+        output->append(redis::NilString());
+      } else {
+        output->append(redis::Integer(it.second.lag));
+      }
+    }
+
+    return Status::OK();
+  }
+
+  Status getConsumerInfo(Server *srv, Connection *conn, std::string *output) {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
+    std::vector<std::pair<std::string, redis::StreamConsumerMetadata>> result_vector;
+    auto s = stream_db.GetConsumerInfo(args_[2], args_[3], result_vector);
+
+    if (!s.ok() && !s.IsNotFound()) {
+      return {Status::RedisExecErr, s.ToString()};
+    }
+
+    if (s.IsNotFound()) {
+      return {Status::RedisExecErr, errNoSuchKey};
+    }
+
+    output->append(redis::MultiLen(result_vector.size()));
+    auto now = util::GetTimeStampMS();
+    for (auto const &it : result_vector) {
+      output->append(redis::MultiLen(8));
+      output->append(redis::BulkString("name"));
+      output->append(redis::BulkString(it.first));
+      output->append(redis::BulkString("pending"));
+      output->append(redis::Integer(it.second.pending_number));
+      output->append(redis::BulkString("idle"));
+      output->append(redis::Integer(now - it.second.last_idle));
+      output->append(redis::BulkString("inactive"));
+      output->append(redis::Integer(now - it.second.last_active));
+    }
+
+    return Status::OK();
+  }
 };
 
 class CommandXRange : public Commander {
@@ -470,13 +609,13 @@ class CommandXRange : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
     if (with_count_ && count_ == 0) {
       *output = redis::NilString();
       return Status::OK();
     }
 
-    redis::Stream stream_db(svr->storage, conn->GetNamespace());
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
 
     redis::StreamRangeOptions options;
     options.reverse = false;
@@ -563,13 +702,13 @@ class CommandXRevRange : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
     if (with_count_ && count_ == 0) {
       *output = redis::NilString();
       return Status::OK();
     }
 
-    redis::Stream stream_db(svr->storage, conn->GetNamespace());
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
 
     redis::StreamRangeOptions options;
     options.reverse = true;
@@ -691,8 +830,8 @@ class CommandXRead : public Commander,
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::Stream stream_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
 
     std::vector<redis::StreamReadResult> results;
 
@@ -727,7 +866,7 @@ class CommandXRead : public Commander,
         return Status::OK();  // No blocking in multi-exec
       }
 
-      return BlockingRead(svr, conn, &stream_db);
+      return BlockingRead(srv, conn, &stream_db);
     }
 
     if (!block_ && results.empty()) {
@@ -755,7 +894,7 @@ class CommandXRead : public Commander,
     return Status::OK();
   }
 
-  Status BlockingRead(Server *svr, Connection *conn, redis::Stream *stream_db) {
+  Status BlockingRead(Server *srv, Connection *conn, redis::Stream *stream_db) {
     if (!with_count_) {
       with_count_ = true;
       count_ = blocked_default_count_;
@@ -773,10 +912,10 @@ class CommandXRead : public Commander,
       }
     }
 
-    svr_ = svr;
+    srv_ = srv;
     conn_ = conn;
 
-    svr_->BlockOnStreams(streams_, ids_, conn_);
+    srv_->BlockOnStreams(streams_, ids_, conn_);
 
     auto bev = conn->GetBufferEvent();
     SetCB(bev);
@@ -807,7 +946,7 @@ class CommandXRead : public Commander,
     conn_->SetCB(bev);
     bufferevent_enable(bev, EV_READ);
 
-    redis::Stream stream_db(svr_->storage, conn_->GetNamespace());
+    redis::Stream stream_db(srv_->storage, conn_->GetNamespace());
 
     std::vector<StreamReadResult> results;
 
@@ -885,7 +1024,7 @@ class CommandXRead : public Commander,
   std::vector<std::string> streams_;
   std::vector<StreamEntryID> ids_;
   std::vector<bool> latest_marks_;
-  Server *svr_ = nullptr;
+  Server *srv_ = nullptr;
   Connection *conn_ = nullptr;
   UniqueEvent timer_;
   uint64_t count_ = 0;
@@ -894,7 +1033,7 @@ class CommandXRead : public Commander,
   bool with_count_ = false;
   bool block_ = false;
 
-  void unblockAll() { svr_->UnblockOnStreams(streams_, conn_); }
+  void unblockAll() { srv_->UnblockOnStreams(streams_, conn_); }
 };
 
 class CommandXTrim : public Commander {
@@ -965,8 +1104,8 @@ class CommandXTrim : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::Stream stream_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
 
     StreamTrimOptions options;
     options.strategy = strategy_;
@@ -1030,8 +1169,8 @@ class CommandXSetId : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *svr, Connection *conn, std::string *output) override {
-    redis::Stream stream_db(svr->storage, conn->GetNamespace());
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
 
     auto s = stream_db.SetId(stream_name_, last_id_, entries_added_, max_deleted_id_);
     if (!s.ok()) {
