@@ -32,6 +32,7 @@
 #include "config_type.h"
 #include "cron.h"
 #include "status.h"
+#include "storage/redis_metadata.h"
 
 // forward declaration
 class Server;
@@ -101,7 +102,7 @@ struct Config {
   int slowlog_log_slower_than = 100000;
   int slowlog_max_len = 128;
   bool daemonize = false;
-  int supervised_mode = kSupervisedNone;
+  SupervisedMode supervised_mode = kSupervisedNone;
   bool slave_readonly = true;
   bool slave_serve_stale_data = true;
   bool slave_empty_db_before_fullsync = false;
@@ -118,12 +119,10 @@ struct Config {
   std::vector<std::string> binds;
   std::string dir;
   std::string db_dir;
-  std::string backup_dir;  // GUARD_BY(backup_mu_)
   std::string backup_sync_dir;
   std::string checkpoint_dir;
   std::string sync_checkpoint_dir;
   std::string log_dir;
-  std::string pidfile;
   std::string db_name;
   std::string masterauth;
   std::string requirepass;
@@ -136,7 +135,7 @@ struct Config {
   CompactionCheckerRange compaction_checker_range{-1, -1};
   int64_t force_compact_file_age;
   int force_compact_file_min_deleted_percentage;
-  std::map<std::string, std::string> tokens;
+  bool repl_namespace_enabled = false;
   std::string replica_announce_ip;
   uint32_t replica_announce_port = 0;
 
@@ -149,6 +148,11 @@ struct Config {
 
   bool redis_cursor_compatible = false;
   int log_retention_days;
+
+  // load_tokens is used to buffer the tokens when loading,
+  // don't use it to authenticate or rewrite the configuration file.
+  std::map<std::string, std::string> load_tokens;
+
   // profiling
   int profiling_sample_ratio = 0;
   int profiling_sample_record_threshold_ms = 0;
@@ -156,6 +160,10 @@ struct Config {
   std::set<std::string> profiling_sample_commands;
   bool profiling_sample_all_commands = false;
 
+  // json
+  int json_max_nesting_depth = 1024;
+  JsonStorageFormat json_storage_format = JsonStorageFormat::JSON;
+
   struct RocksDB {
     int block_size;
     bool cache_index_and_filter_blocks;
@@ -181,7 +189,7 @@ struct Config {
     int level0_slowdown_writes_trigger;
     int level0_stop_writes_trigger;
     int level0_file_num_compaction_trigger;
-    int compression;
+    rocksdb::CompressionType compression;
     bool disable_auto_compactions;
     bool enable_blob_files;
     int min_blob_size;
@@ -193,6 +201,7 @@ struct Config {
     bool level_compaction_dynamic_level_bytes;
     int max_background_jobs;
     bool rate_limiter_auto_tuned;
+    bool avoid_unnecessary_blocking_io = true;
 
     struct WriteOptions {
       bool sync;
@@ -210,19 +219,21 @@ struct Config {
   mutable std::mutex backup_mu;
 
   std::string NodesFilePath() const;
-  Status Rewrite();
+  Status Rewrite(const std::map<std::string, std::string> &tokens);
   Status Load(const CLIOptions &path);
   void Get(const std::string &key, std::vector<std::string> *values) const;
-  Status Set(Server *svr, std::string key, const std::string &value);
+  Status Set(Server *srv, std::string key, const std::string &value);
   void SetMaster(const std::string &host, uint32_t port);
   void ClearMaster();
-  Status GetNamespace(const std::string &ns, std::string *token) const;
-  Status AddNamespace(const std::string &ns, const std::string &token);
-  Status SetNamespace(const std::string &ns, const std::string &token);
-  Status DelNamespace(const std::string &ns);
+  bool IsSlave() const { return !master_host.empty(); }
+  bool HasConfigFile() const { return !path_.empty(); }
+  std::string GetBackupDir() const { return backup_dir_.empty() ? dir + "/backup" : backup_dir_; }
+  std::string GetPidFile() const { return pidfile_.empty() ? dir + "/kvrocks.pid" : pidfile_; }
 
  private:
   std::string path_;
+  std::string backup_dir_;  // GUARD_BY(backup_mu_)
+  std::string pidfile_;
   std::string binds_str_;
   std::string slaveof_;
   std::string compact_cron_str_;
@@ -236,6 +247,6 @@ struct Config {
   void initFieldCallback();
   Status parseConfigFromPair(const std::pair<std::string, std::string> &input, int line_number);
   Status parseConfigFromString(const std::string &input, int line_number);
+  bool checkFieldValueIsDefault(const std::string &key, const std::string &value) const;
   Status finish();
-  static Status isNamespaceLegal(const std::string &ns);
 };
