@@ -44,6 +44,7 @@
 #include "storage/compaction_checker.h"
 #include "storage/redis_db.h"
 #include "storage/scripting.h"
+#include "storage/storage.h"
 #include "string_util.h"
 #include "thread_util.h"
 #include "time_util.h"
@@ -53,9 +54,9 @@
 constexpr const char *REDIS_VERSION = "4.0.0";
 
 Server::Server(engine::Storage *storage, Config *config)
-    : storage(storage), start_time_(util::GetTimeStamp()), config_(config) {
+    : storage(storage), start_time_(util::GetTimeStamp()), config_(config), namespace_(storage) {
   // init commands stats here to prevent concurrent insert, and cause core
-  auto commands = redis::GetOriginalCommands();
+  auto commands = redis::CommandTable::GetOriginal();
   for (const auto &iter : *commands) {
     stats.commands_stats[iter.first].calls = 0;
     stats.commands_stats[iter.first].latency = 0;
@@ -110,10 +111,12 @@ Server::~Server() {
       break;
     }
   }
-  // Manually reset workers here to avoid accessing the conn_ctxs_ after it's freed
+
   for (auto &worker_thread : worker_threads_) {
     worker_thread.reset();
   }
+  cleanupExitedWorkerThreads(true /* force */);
+  CleanupExitedSlaves();
 
   lua::DestroyState(lua_);
 }
@@ -131,12 +134,16 @@ Server::~Server() {
 //     - feed-replica-data-info: generate checkpoint and send files list when full sync
 //     - feed-replica-file: send SST files when slaves ask for full sync
 Status Server::Start() {
+  auto s = namespace_.LoadAndRewrite();
+  if (!s.IsOK()) {
+    return s;
+  }
   if (!config_->master_host.empty()) {
-    Status s = AddMaster(config_->master_host, static_cast<uint32_t>(config_->master_port), false);
+    s = AddMaster(config_->master_host, static_cast<uint32_t>(config_->master_port), false);
     if (!s.IsOK()) return s;
   } else {
     // Generate new replication id if not a replica
-    auto s = storage->ShiftReplId();
+    s = storage->ShiftReplId();
     if (!s.IsOK()) {
       return s.Prefixed("failed to shift replication id");
     }
@@ -219,7 +226,7 @@ void Server::Stop() {
   slaveof_mu_.unlock();
 
   for (const auto &worker : worker_threads_) {
-    worker->Stop();
+    worker->Stop(0 /* immediately terminate  */);
   }
 
   rocksdb::CancelAllBackgroundWork(storage->GetDB(), true);
@@ -227,10 +234,6 @@ void Server::Stop() {
 }
 
 void Server::Join() {
-  for (const auto &worker : worker_threads_) {
-    worker->Join();
-  }
-
   if (auto s = util::ThreadJoin(cron_thread_); !s) {
     LOG(WARNING) << "Cron thread operation failed: " << s.Msg();
   }
@@ -240,6 +243,9 @@ void Server::Join() {
   if (auto s = task_runner_.Join(); !s) {
     LOG(WARNING) << s.Msg();
   }
+  for (const auto &worker : worker_threads_) {
+    worker->Join();
+  }
 }
 
 Status Server::AddMaster(const std::string &host, uint32_t port, bool force_reconnect) {
@@ -412,8 +418,6 @@ void Server::SubscribeChannel(const std::string &channel, redis::Connection *con
   std::lock_guard<std::mutex> guard(pubsub_channels_mu_);
 
   auto conn_ctx = ConnContext(conn->Owner(), conn->GetFD());
-  conn_ctxs_[conn_ctx] = true;
-
   if (auto iter = pubsub_channels_.find(channel); iter == pubsub_channels_.end()) {
     pubsub_channels_.emplace(channel, std::list<ConnContext>{conn_ctx});
   } else {
@@ -431,7 +435,6 @@ void Server::UnsubscribeChannel(const std::string &channel, redis::Connection *c
 
   for (const auto &conn_ctx : iter->second) {
     if (conn->GetFD() == conn_ctx.fd && conn->Owner() == conn_ctx.owner) {
-      conn_ctxs_.erase(conn_ctx);
       iter->second.remove(conn_ctx);
       if (iter->second.empty()) {
         pubsub_channels_.erase(iter);
@@ -468,8 +471,6 @@ void Server::PSubscribeChannel(const std::string &pattern, redis::Connection *co
   std::lock_guard<std::mutex> guard(pubsub_channels_mu_);
 
   auto conn_ctx = ConnContext(conn->Owner(), conn->GetFD());
-  conn_ctxs_[conn_ctx] = true;
-
   if (auto iter = pubsub_patterns_.find(pattern); iter == pubsub_patterns_.end()) {
     pubsub_patterns_.emplace(pattern, std::list<ConnContext>{conn_ctx});
   } else {
@@ -487,7 +488,6 @@ void Server::PUnsubscribeChannel(const std::string &pattern, redis::Connection *
 
   for (const auto &conn_ctx : iter->second) {
     if (conn->GetFD() == conn_ctx.fd && conn->Owner() == conn_ctx.owner) {
-      conn_ctxs_.erase(conn_ctx);
       iter->second.remove(conn_ctx);
       if (iter->second.empty()) {
         pubsub_patterns_.erase(iter);
@@ -501,7 +501,6 @@ void Server::BlockOnKey(const std::string &key, redis::Connection *conn) {
   std::lock_guard<std::mutex> guard(blocking_keys_mu_);
 
   auto conn_ctx = ConnContext(conn->Owner(), conn->GetFD());
-  conn_ctxs_[conn_ctx] = true;
 
   if (auto iter = blocking_keys_.find(key); iter == blocking_keys_.end()) {
     blocking_keys_.emplace(key, std::list<ConnContext>{conn_ctx});
@@ -522,7 +521,6 @@ void Server::UnblockOnKey(const std::string &key, redis::Connection *conn) {
 
   for (const auto &conn_ctx : iter->second) {
     if (conn->GetFD() == conn_ctx.fd && conn->Owner() == conn_ctx.owner) {
-      conn_ctxs_.erase(conn_ctx);
       iter->second.remove(conn_ctx);
       if (iter->second.empty()) {
         blocking_keys_.erase(iter);
@@ -591,7 +589,6 @@ void Server::WakeupBlockingConns(const std::string &key, size_t n_conns) {
     if (!s.IsOK()) {
       LOG(ERROR) << "[server] Failed to enable write event on blocked client " << conn_ctx.fd << ": " << s.Msg();
     }
-    conn_ctxs_.erase(conn_ctx);
     iter->second.pop_front();
   }
 }
@@ -742,6 +739,11 @@ void Server::cron() {
       storage->SetDBInRetryableIOError(false);
     }
 
+    // check if we need to clean up exited worker threads every 5s
+    if (counter != 0 && counter % 50 == 0) {
+      cleanupExitedWorkerThreads(false);
+    }
+
     CleanupExitedSlaves();
     recordInstantaneousMetrics();
   }
@@ -782,17 +784,21 @@ void Server::GetRocksDBInfo(std::string *info) {
                   << "\r\n";
     db->GetMapProperty(cf_handle, rocksdb::DB::Properties::kCFStats, &cf_stats_map);
     string_stream << "level0_file_limit_slowdown[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["io_stalls.level0_slowdown"] << "\r\n";
+                  << "]:" << cf_stats_map["l0-file-count-limit-delays"] << "\r\n";
     string_stream << "level0_file_limit_stop[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["io_stalls.level0_numfiles"] << "\r\n";
+                  << "]:" << cf_stats_map["l0-file-count-limit-stops"] << "\r\n";
     string_stream << "pending_compaction_bytes_slowdown[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["io_stalls.slowdown_for_pending_compaction_bytes"] << "\r\n";
+                  << "]:" << cf_stats_map["pending-compaction-bytes-delays"] << "\r\n";
     string_stream << "pending_compaction_bytes_stop[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["io_stalls.stop_for_pending_compaction_bytes"] << "\r\n";
+                  << "]:" << cf_stats_map["pending-compaction-bytes-stops"] << "\r\n";
+    string_stream << "level0_file_limit_stop_with_ongoing_compaction[" << cf_handle->GetName()
+                  << "]:" << cf_stats_map["cf-l0-file-count-limit-stops-with-ongoing-compaction"] << "\r\n";
+    string_stream << "level0_file_limit_slowdown_with_ongoing_compaction[" << cf_handle->GetName()
+                  << "]:" << cf_stats_map["cf-l0-file-count-limit-delays-with-ongoing-compaction"] << "\r\n";
     string_stream << "memtable_count_limit_slowdown[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["io_stalls.memtable_slowdown"] << "\r\n";
+                  << "]:" << cf_stats_map["memtable-limit-delays"] << "\r\n";
     string_stream << "memtable_count_limit_stop[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["io_stalls.memtable_compaction"] << "\r\n";
+                  << "]:" << cf_stats_map["memtable-limit-stops"] << "\r\n";
   }
   string_stream << "all_mem_tables:" << memtable_sizes << "\r\n";
   string_stream << "cur_mem_tables:" << cur_memtable_sizes << "\r\n";
@@ -839,6 +845,8 @@ void Server::GetServerInfo(std::string *info) {
   string_stream << "redis_version:" << REDIS_VERSION << "\r\n";
   string_stream << "git_sha1:" << GIT_COMMIT << "\r\n";
   string_stream << "kvrocks_git_sha1:" << GIT_COMMIT << "\r\n";
+  string_stream << "redis_mode:" << (config_->cluster_enabled ? "cluster" : "standalone") << "\r\n";
+  string_stream << "kvrocks_mode:" << (config_->cluster_enabled ? "cluster" : "standalone") << "\r\n";
   string_stream << "os:" << name.sysname << " " << name.release << " " << name.machine << "\r\n";
 #ifdef __GNUC__
   string_stream << "gcc_version:" << __GNUC__ << "." << __GNUC_MINOR__ << "." << __GNUC_PATCHLEVEL__ << "\r\n";
@@ -1252,7 +1260,7 @@ Status Server::AsyncCompactDB(const std::string &begin_key, const std::string &e
     if (!begin_key.empty()) begin = std::make_unique<Slice>(begin_key);
     if (!end_key.empty()) end = std::make_unique<Slice>(end_key);
 
-    auto s = storage->Compact(begin.get(), end.get());
+    auto s = storage->Compact(nullptr, begin.get(), end.get());
     if (!s.ok()) {
       LOG(ERROR) << "[task runner] Failed to do compaction: " << s.ToString();
     }
@@ -1393,7 +1401,7 @@ Status Server::autoResizeBlockAndSST() {
     config_->rocks_db.block_size = block_size;
   }
 
-  auto s = config_->Rewrite();
+  auto s = config_->Rewrite(namespace_.List());
   LOG(INFO) << "[server] Rewrite config, result: " << s.Msg();
 
   return Status::OK();
@@ -1504,7 +1512,7 @@ ReplState Server::GetReplicationState() {
 Status Server::LookupAndCreateCommand(const std::string &cmd_name, std::unique_ptr<redis::Commander> *cmd) {
   if (cmd_name.empty()) return {Status::RedisUnknownCmd};
 
-  auto commands = redis::GetCommands();
+  auto commands = redis::CommandTable::Get();
   auto cmd_iter = commands->find(util::ToLower(cmd_name));
   if (cmd_iter == commands->end()) {
     return {Status::RedisUnknownCmd};
@@ -1527,7 +1535,7 @@ Status Server::ScriptExists(const std::string &sha) {
 }
 
 Status Server::ScriptGet(const std::string &sha, std::string *body) const {
-  std::string func_name = engine::kLuaFunctionPrefix + sha;
+  std::string func_name = engine::kLuaFuncSHAPrefix + sha;
   auto cf = storage->GetCFHandle(engine::kPropagateColumnFamilyName);
   auto s = storage->Get(rocksdb::ReadOptions(), cf, func_name, body);
   if (!s.ok()) {
@@ -1537,10 +1545,40 @@ Status Server::ScriptGet(const std::string &sha, std::string *body) const {
 }
 
 Status Server::ScriptSet(const std::string &sha, const std::string &body) const {
-  std::string func_name = engine::kLuaFunctionPrefix + sha;
+  std::string func_name = engine::kLuaFuncSHAPrefix + sha;
   return storage->WriteToPropagateCF(func_name, body);
 }
 
+Status Server::FunctionGetCode(const std::string &lib, std::string *code) const {
+  std::string func_name = engine::kLuaLibCodePrefix + lib;
+  auto cf = storage->GetCFHandle(engine::kPropagateColumnFamilyName);
+  auto s = storage->Get(rocksdb::ReadOptions(), cf, func_name, code);
+  if (!s.ok()) {
+    return {s.IsNotFound() ? Status::NotFound : Status::NotOK, s.ToString()};
+  }
+  return Status::OK();
+}
+
+Status Server::FunctionGetLib(const std::string &func, std::string *lib) const {
+  std::string func_name = engine::kLuaFuncLibPrefix + func;
+  auto cf = storage->GetCFHandle(engine::kPropagateColumnFamilyName);
+  auto s = storage->Get(rocksdb::ReadOptions(), cf, func_name, lib);
+  if (!s.ok()) {
+    return {s.IsNotFound() ? Status::NotFound : Status::NotOK, s.ToString()};
+  }
+  return Status::OK();
+}
+
+Status Server::FunctionSetCode(const std::string &lib, const std::string &code) const {
+  std::string func_name = engine::kLuaLibCodePrefix + lib;
+  return storage->WriteToPropagateCF(func_name, code);
+}
+
+Status Server::FunctionSetLib(const std::string &func, const std::string &lib) const {
+  std::string func_name = engine::kLuaFuncLibPrefix + func;
+  return storage->WriteToPropagateCF(func_name, lib);
+}
+
 void Server::ScriptReset() {
   auto lua = lua_.exchange(lua::CreateState(this));
   lua::DestroyState(lua);
@@ -1645,6 +1683,73 @@ void Server::AdjustOpenFilesLimit() {
   }
 }
 
+void Server::AdjustWorkerThreads() {
+  auto new_worker_threads = static_cast<size_t>(config_->workers);
+  if (new_worker_threads == worker_threads_.size()) {
+    return;
+  }
+  size_t delta = 0;
+  if (new_worker_threads > worker_threads_.size()) {
+    delta = new_worker_threads - worker_threads_.size();
+    increaseWorkerThreads(delta);
+    LOG(INFO) << "[server] Increase worker threads from " << worker_threads_.size() << " to " << new_worker_threads;
+    return;
+  }
+
+  delta = worker_threads_.size() - new_worker_threads;
+  LOG(INFO) << "[server] Decrease worker threads from " << worker_threads_.size() << " to " << new_worker_threads;
+  decreaseWorkerThreads(delta);
+}
+
+void Server::increaseWorkerThreads(size_t delta) {
+  for (size_t i = 0; i < delta; i++) {
+    auto worker = std::make_unique<Worker>(this, config_);
+    auto worker_thread = std::make_unique<WorkerThread>(std::move(worker));
+    worker_thread->Start();
+    worker_threads_.emplace_back(std::move(worker_thread));
+  }
+}
+
+void Server::decreaseWorkerThreads(size_t delta) {
+  auto current_worker_threads = worker_threads_.size();
+  DCHECK(current_worker_threads > delta);
+  auto remain_worker_threads = current_worker_threads - delta;
+  for (size_t i = remain_worker_threads; i < current_worker_threads; i++) {
+    // Unix socket will be listening on the first worker,
+    // so it MUST remove workers from the end of the vector.
+    // Otherwise, the unix socket will be closed.
+    auto worker_thread = std::move(worker_threads_.back());
+    worker_threads_.pop_back();
+    // Migrate connections to other workers before stopping the worker,
+    // we use round-robin to choose the target worker here.
+    auto connections = worker_thread->GetWorker()->GetConnections();
+    for (const auto &iter : connections) {
+      auto target_worker = worker_threads_[iter.first % remain_worker_threads]->GetWorker();
+      worker_thread->GetWorker()->MigrateConnection(target_worker, iter.second);
+    }
+    worker_thread->Stop(10 /* graceful timeout */);
+    // Don't join the worker thread here, because it may join itself.
+    recycle_worker_threads_.push(std::move(worker_thread));
+  }
+}
+
+void Server::cleanupExitedWorkerThreads(bool force) {
+  std::unique_ptr<WorkerThread> worker_thread = nullptr;
+  auto total = recycle_worker_threads_.unsafe_size();
+  for (size_t i = 0; i < total; i++) {
+    if (!recycle_worker_threads_.try_pop(worker_thread)) {
+      break;
+    }
+    if (worker_thread->IsTerminated() || force) {
+      worker_thread->Join();
+      worker_thread.reset();
+    } else {
+      // Push the worker thread back to the queue if it's still running.
+      recycle_worker_threads_.push(std::move(worker_thread));
+    }
+  }
+}
+
 std::string ServerLogData::Encode() const {
   if (type_ == kReplIdLog) {
     return std::string(1, kReplIdTag) + " " + content_;
