@@ -21,6 +21,7 @@
 #pragma once
 
 #include <inttypes.h>
+#include <tbb/concurrent_vector.h>
 
 #include <array>
 #include <atomic>
@@ -42,6 +43,7 @@
 #include "cluster/slot_migrate.h"
 #include "commands/commander.h"
 #include "lua.hpp"
+#include "namespace.h"
 #include "server/redis_connection.h"
 #include "stats/log_collector.h"
 #include "stats/stats.h"
@@ -177,6 +179,7 @@ class Server {
   Config *GetConfig() { return config_; }
   static Status LookupAndCreateCommand(const std::string &cmd_name, std::unique_ptr<redis::Commander> *cmd);
   void AdjustOpenFilesLimit();
+  void AdjustWorkerThreads();
 
   Status AddMaster(const std::string &host, uint32_t port, bool force_reconnect);
   Status RemoveMaster();
@@ -255,6 +258,11 @@ class Server {
   void ScriptReset();
   Status ScriptFlush();
 
+  Status FunctionGetCode(const std::string &lib, std::string *code) const;
+  Status FunctionGetLib(const std::string &func, std::string *lib) const;
+  Status FunctionSetCode(const std::string &lib, const std::string &code) const;
+  Status FunctionSetLib(const std::string &func, const std::string &lib) const;
+
   Status Propagate(const std::string &channel, const std::vector<std::string> &tokens) const;
   Status ExecPropagatedCommand(const std::vector<std::string> &tokens);
   Status ExecPropagateScriptCommand(const std::vector<std::string> &tokens);
@@ -282,6 +290,7 @@ class Server {
   static bool IsWatchedKeysModified(redis::Connection *conn);
   void ResetWatchedKeys(redis::Connection *conn);
   std::list<std::pair<std::string, uint32_t>> GetSlaveHostAndPort();
+  Namespace *GetNamespace() { return &namespace_; }
 
 #ifdef ENABLE_OPENSSL
   UniqueSSLContext ssl_ctx;
@@ -294,6 +303,9 @@ class Server {
   Status autoResizeBlockAndSST();
   void updateWatchedKeysFromRange(const std::vector<std::string> &args, const redis::CommandKeyRange &range);
   void updateAllWatchedKeys();
+  void increaseWorkerThreads(size_t delta);
+  void decreaseWorkerThreads(size_t delta);
+  void cleanupExitedWorkerThreads(bool force);
 
   std::atomic<bool> stop_ = false;
   std::atomic<bool> is_loading_ = false;
@@ -320,6 +332,9 @@ class Server {
   std::list<std::unique_ptr<FeedSlaveThread>> slave_threads_;
   std::atomic<int> fetch_file_threads_num_ = 0;
 
+  // namespace
+  Namespace namespace_;
+
   // Some jobs to operate DB should be unique
   std::mutex db_job_mu_;
   bool db_compacting_ = false;
@@ -333,7 +348,6 @@ class Server {
   LogCollector<SlowEntry> slow_log_;
   LogCollector<PerfEntry> perf_log_;
 
-  std::map<ConnContext, bool> conn_ctxs_;
   std::map<std::string, std::list<ConnContext>> pubsub_channels_;
   std::map<std::string, std::list<ConnContext>> pubsub_patterns_;
   std::mutex pubsub_channels_mu_;
@@ -352,6 +366,7 @@ class Server {
   TaskRunner task_runner_;
   std::vector<std::unique_ptr<WorkerThread>> worker_threads_;
   std::unique_ptr<ReplicationThread> replication_thread_;
+  tbb::concurrent_queue<std::unique_ptr<WorkerThread>> recycle_worker_threads_;
 
   // memory
   std::atomic<int64_t> memory_startup_use_ = 0;
