@@ -52,7 +52,7 @@
 #include "server.h"
 #include "storage/scripting.h"
 
-Worker::Worker(Server *svr, Config *config) : svr(svr), base_(event_base_new()) {
+Worker::Worker(Server *srv, Config *config) : srv(srv), base_(event_base_new()) {
   if (!base_) throw std::runtime_error{"event base failed to be created"};
 
   timer_.reset(NewEvent(base_, -1, EV_PERSIST));
@@ -72,7 +72,7 @@ Worker::Worker(Server *svr, Config *config) : svr(svr), base_(event_base_new())
       LOG(INFO) << "[worker] Listening on: " << bind << ":" << *port;
     }
   }
-  lua_ = lua::CreateState(svr, true);
+  lua_ = lua::CreateState(srv, true);
 }
 
 Worker::~Worker() {
@@ -101,7 +101,7 @@ Worker::~Worker() {
 }
 
 void Worker::TimerCB(int, int16_t events) {
-  auto config = svr->GetConfig();
+  auto config = srv->GetConfig();
   if (config->timeout == 0) return;
   KickoutIdleClients(config->timeout);
 }
@@ -131,8 +131,8 @@ void Worker::newTCPConnection(evconnlistener *listener, evutil_socket_t fd, sock
   bufferevent *bev = nullptr;
   ssl_st *ssl = nullptr;
 #ifdef ENABLE_OPENSSL
-  if (uint32_t(local_port) == svr->GetConfig()->tls_port) {
-    ssl = SSL_new(svr->ssl_ctx.get());
+  if (uint32_t(local_port) == srv->GetConfig()->tls_port) {
+    ssl = SSL_new(srv->ssl_ctx.get());
     if (!ssl) {
       LOG(ERROR) << "Failed to construct SSL structure for new connection: " << SSLErrors{};
       evutil_closesocket(fd);
@@ -157,7 +157,7 @@ void Worker::newTCPConnection(evconnlistener *listener, evutil_socket_t fd, sock
     return;
   }
 #ifdef ENABLE_OPENSSL
-  if (uint32_t(local_port) == svr->GetConfig()->tls_port) {
+  if (uint32_t(local_port) == srv->GetConfig()->tls_port) {
     bufferevent_openssl_set_allow_dirty_shutdown(bev, 1);
   }
 #endif
@@ -187,7 +187,7 @@ void Worker::newTCPConnection(evconnlistener *listener, evutil_socket_t fd, sock
 }
 
 void Worker::newUnixSocketConnection(evconnlistener *listener, evutil_socket_t fd, sockaddr *address, int socklen) {
-  DLOG(INFO) << "[worker] New connection: fd=" << fd << " from unixsocket: " << svr->GetConfig()->unixsocket
+  DLOG(INFO) << "[worker] New connection: fd=" << fd << " from unixsocket: " << srv->GetConfig()->unixsocket
              << " thread #" << tid_;
   event_base *base = evconnlistener_get_base(listener);
   auto ev_thread_safe_flags =
@@ -209,7 +209,7 @@ void Worker::newUnixSocketConnection(evconnlistener *listener, evutil_socket_t f
     return;
   }
 
-  conn->SetAddr(svr->GetConfig()->unixsocket, 0);
+  conn->SetAddr(srv->GetConfig()->unixsocket, 0);
   if (rate_limit_group_) {
     bufferevent_add_to_rate_limit_group(bev, rate_limit_group_);
   }
@@ -252,7 +252,8 @@ Status Worker::listenTCP(const std::string &host, uint32_t port, int backlog) {
     }
 
     evutil_make_socket_nonblocking(fd);
-    auto lev = NewEvconnlistener<&Worker::newTCPConnection>(base_, LEV_OPT_CLOSE_ON_FREE, backlog, fd);
+    auto lev =
+        NewEvconnlistener<&Worker::newTCPConnection>(base_, LEV_OPT_THREADSAFE | LEV_OPT_CLOSE_ON_FREE, backlog, fd);
     listen_events_.emplace_back(lev);
   }
 
@@ -273,7 +274,7 @@ Status Worker::ListenUnixSocket(const std::string &path, int perm, int backlog)
     return {Status::NotOK, evutil_socket_error_to_string(EVUTIL_SOCKET_ERROR())};
   }
 
-  if (bind(fd, (struct sockaddr *)&sa, sizeof(sa)) < 0) {
+  if (bind(fd, (sockaddr *)&sa, sizeof(sa)) < 0) {
     return {Status::NotOK, evutil_socket_error_to_string(EVUTIL_SOCKET_ERROR())};
   }
 
@@ -292,14 +293,22 @@ void Worker::Run(std::thread::id tid) {
   if (event_base_dispatch(base_) != 0) {
     LOG(ERROR) << "[worker] Failed to run server, err: " << strerror(errno);
   }
+  is_terminated_ = true;
 }
 
-void Worker::Stop() {
-  event_base_loopbreak(base_);
+void Worker::Stop(uint32_t wait_seconds) {
   for (const auto &lev : listen_events_) {
     // It's unnecessary to close the listener fd since we have set the LEV_OPT_CLOSE_ON_FREE flag
     evconnlistener_free(lev);
   }
+  // wait_seconds == 0 means stop immediately, or it will wait N seconds
+  // for the worker to process the remaining requests before stopping.
+  if (wait_seconds > 0) {
+    timeval tv = {wait_seconds, 0};
+    event_base_loopexit(base_, &tv);
+  } else {
+    event_base_loopbreak(base_);
+  }
 }
 
 Status Worker::AddConnection(redis::Connection *c) {
@@ -309,14 +318,14 @@ Status Worker::AddConnection(redis::Connection *c) {
     return {Status::NotOK, "connection was exists"};
   }
 
-  int max_clients = svr->GetConfig()->maxclients;
-  if (svr->IncrClientNum() >= max_clients) {
-    svr->DecrClientNum();
+  int max_clients = srv->GetConfig()->maxclients;
+  if (srv->IncrClientNum() >= max_clients) {
+    srv->DecrClientNum();
     return {Status::NotOK, "max number of clients reached"};
   }
 
   conns_.emplace(c->GetFD(), c);
-  uint64_t id = svr->GetClientID();
+  uint64_t id = srv->GetClientID();
   c->SetID(id);
 
   return Status::OK();
@@ -330,20 +339,51 @@ redis::Connection *Worker::removeConnection(int fd) {
   if (iter != conns_.end()) {
     conn = iter->second;
     conns_.erase(iter);
-    svr->DecrClientNum();
+    srv->DecrClientNum();
   }
 
   iter = monitor_conns_.find(fd);
   if (iter != monitor_conns_.end()) {
     conn = iter->second;
     monitor_conns_.erase(iter);
-    svr->DecrClientNum();
-    svr->DecrMonitorClientNum();
+    srv->DecrClientNum();
+    srv->DecrMonitorClientNum();
   }
 
   return conn;
 }
 
+// MigrateConnection moves the connection to another worker
+// when reducing the number of workers.
+//
+// To make it simple, we would close the connection if it's
+// blocked on a key or stream.
+void Worker::MigrateConnection(Worker *target, redis::Connection *conn) {
+  if (!target || !conn) return;
+
+  auto bev = conn->GetBufferEvent();
+  // disable read/write event to prevent the connection from being processed during migration
+  bufferevent_disable(bev, EV_READ | EV_WRITE);
+  // We cannot migrate the connection if it has a running command
+  // since it will cause data race since the old worker may still process the command.
+  if (!conn->CanMigrate()) {
+    // Need to enable read/write event again since we disabled them before
+    bufferevent_enable(bev, EV_READ | EV_WRITE);
+    return;
+  }
+
+  // remove the connection from current worker
+  DetachConnection(conn);
+  if (!target->AddConnection(conn).IsOK()) {
+    conn->Close();
+    return;
+  }
+  bufferevent_base_set(target->base_, bev);
+  conn->SetCB(bev);
+  bufferevent_enable(bev, EV_READ | EV_WRITE);
+  conn->SetOwner(target);
+}
+
 void Worker::DetachConnection(redis::Connection *conn) {
   if (!conn) return;
 
@@ -362,7 +402,7 @@ void Worker::FreeConnection(redis::Connection *conn) {
   if (!conn) return;
 
   removeConnection(conn->GetFD());
-  svr->ResetWatchedKeys(conn);
+  srv->ResetWatchedKeys(conn);
   if (rate_limit_group_) {
     bufferevent_remove_from_rate_limit_group(conn->GetBufferEvent());
   }
@@ -378,15 +418,15 @@ void Worker::FreeConnectionByID(int fd, uint64_t id) {
     }
     delete iter->second;
     conns_.erase(iter);
-    svr->DecrClientNum();
+    srv->DecrClientNum();
   }
 
   iter = monitor_conns_.find(fd);
   if (iter != monitor_conns_.end() && iter->second->GetID() == id) {
     delete iter->second;
     monitor_conns_.erase(iter);
-    svr->DecrClientNum();
-    svr->DecrMonitorClientNum();
+    srv->DecrClientNum();
+    srv->DecrMonitorClientNum();
   }
 }
 
@@ -420,7 +460,7 @@ void Worker::BecomeMonitorConn(redis::Connection *conn) {
     conns_.erase(conn->GetFD());
     monitor_conns_[conn->GetFD()] = conn;
   }
-  svr->IncrMonitorClientNum();
+  srv->IncrMonitorClientNum();
   conn->EnableFlag(redis::Connection::kMonitor);
 }
 
@@ -515,7 +555,7 @@ void WorkerThread::Start() {
   LOG(INFO) << "[worker] Thread #" << t_.get_id() << " started";
 }
 
-void WorkerThread::Stop() { worker_->Stop(); }
+void WorkerThread::Stop(uint32_t wait_seconds) { worker_->Stop(wait_seconds); }
 
 void WorkerThread::Join() {
   if (auto s = util::ThreadJoin(t_); !s) {
