@@ -34,7 +34,10 @@ class Database {
   static constexpr uint64_t RANDOM_KEY_SCAN_LIMIT = 60;
 
   explicit Database(engine::Storage *storage, std::string ns = "");
+  [[nodiscard]] static rocksdb::Status ParseMetadata(RedisType type, Slice *bytes, Metadata *metadata);
   [[nodiscard]] rocksdb::Status GetMetadata(RedisType type, const Slice &ns_key, Metadata *metadata);
+  [[nodiscard]] rocksdb::Status GetMetadata(RedisType type, const Slice &ns_key, std::string *raw_value,
+                                            Metadata *metadata, Slice *rest);
   [[nodiscard]] rocksdb::Status GetRawMetadata(const Slice &ns_key, std::string *bytes);
   [[nodiscard]] rocksdb::Status GetRawMetadataByUserKey(const Slice &user_key, std::string *bytes);
   [[nodiscard]] rocksdb::Status Expire(const Slice &user_key, uint64_t timestamp);
@@ -59,6 +62,7 @@ class Database {
   [[nodiscard]] rocksdb::Status ClearKeysOfSlot(const rocksdb::Slice &ns, int slot);
   [[nodiscard]] rocksdb::Status GetSlotKeysInfo(int slot, std::map<int, uint64_t> *slotskeys,
                                                 std::vector<std::string> *keys, int count);
+  [[nodiscard]] rocksdb::Status KeyExist(const std::string &key);
 
  protected:
   engine::Storage *storage_;
@@ -66,20 +70,20 @@ class Database {
   std::string namespace_;
 
   friend class LatestSnapShot;
-  class LatestSnapShot {
-   public:
-    explicit LatestSnapShot(engine::Storage *storage)
-        : storage_(storage), snapshot_(storage_->GetDB()->GetSnapshot()) {}
-    ~LatestSnapShot() { storage_->GetDB()->ReleaseSnapshot(snapshot_); }
-    const rocksdb::Snapshot *GetSnapShot() const { return snapshot_; }
+};
 
-    LatestSnapShot(const LatestSnapShot &) = delete;
-    LatestSnapShot &operator=(const LatestSnapShot &) = delete;
+class LatestSnapShot {
+ public:
+  explicit LatestSnapShot(engine::Storage *storage) : storage_(storage), snapshot_(storage_->GetDB()->GetSnapshot()) {}
+  ~LatestSnapShot() { storage_->GetDB()->ReleaseSnapshot(snapshot_); }
+  const rocksdb::Snapshot *GetSnapShot() const { return snapshot_; }
 
-   private:
-    engine::Storage *storage_ = nullptr;
-    const rocksdb::Snapshot *snapshot_ = nullptr;
-  };
+  LatestSnapShot(const LatestSnapShot &) = delete;
+  LatestSnapShot &operator=(const LatestSnapShot &) = delete;
+
+ private:
+  engine::Storage *storage_ = nullptr;
+  const rocksdb::Snapshot *snapshot_ = nullptr;
 };
 
 class SubKeyScanner : public redis::Database {
