@@ -119,7 +119,7 @@ void Storage::SetBlobDB(rocksdb::ColumnFamilyOptions *cf_options) {
   cf_options->enable_blob_files = config_->rocks_db.enable_blob_files;
   cf_options->min_blob_size = config_->rocks_db.min_blob_size;
   cf_options->blob_file_size = config_->rocks_db.blob_file_size;
-  cf_options->blob_compression_type = static_cast<rocksdb::CompressionType>(config_->rocks_db.compression);
+  cf_options->blob_compression_type = config_->rocks_db.compression;
   cf_options->enable_blob_garbage_collection = config_->rocks_db.enable_blob_garbage_collection;
   // Use 100.0 to force converting blob_garbage_collection_age_cutoff to double
   cf_options->blob_garbage_collection_age_cutoff = config_->rocks_db.blob_garbage_collection_age_cutoff / 100.0;
@@ -149,7 +149,7 @@ rocksdb::Options Storage::InitRocksDBOptions() {
     if (i < 2) {
       options.compression_per_level[i] = rocksdb::CompressionType::kNoCompression;
     } else {
-      options.compression_per_level[i] = static_cast<rocksdb::CompressionType>(config_->rocks_db.compression);
+      options.compression_per_level[i] = config_->rocks_db.compression;
     }
   }
   if (config_->rocks_db.row_cache_size) {
@@ -186,6 +186,9 @@ rocksdb::Options Storage::InitRocksDBOptions() {
   options.level_compaction_dynamic_level_bytes = config_->rocks_db.level_compaction_dynamic_level_bytes;
   options.max_background_jobs = config_->rocks_db.max_background_jobs;
 
+  // avoid blocking io on iteration
+  // see https://github.com/facebook/rocksdb/wiki/IO#avoid-blocking-io
+  options.avoid_unnecessary_blocking_io = config_->rocks_db.avoid_unnecessary_blocking_io;
   return options;
 }
 
@@ -336,7 +339,7 @@ Status Storage::Open(bool read_only) {
 Status Storage::CreateBackup() {
   LOG(INFO) << "[storage] Start to create new backup";
   std::lock_guard<std::mutex> lg(config_->backup_mu);
-  std::string task_backup_dir = config_->backup_dir;
+  std::string task_backup_dir = config_->GetBackupDir();
 
   std::string tmpdir = task_backup_dir + ".tmp";
   // Maybe there is a dirty tmp checkpoint, try to clean it
@@ -486,7 +489,7 @@ void Storage::EmptyDB() {
 void Storage::PurgeOldBackups(uint32_t num_backups_to_keep, uint32_t backup_max_keep_hours) {
   time_t now = util::GetTimeStamp();
   std::lock_guard<std::mutex> lg(config_->backup_mu);
-  std::string task_backup_dir = config_->backup_dir;
+  std::string task_backup_dir = config_->GetBackupDir();
 
   // Return if there is no backup
   auto s = env_->FileExists(task_backup_dir);
@@ -528,6 +531,19 @@ rocksdb::Status Storage::Get(const rocksdb::ReadOptions &options, rocksdb::Colum
   return db_->Get(options, column_family, key, value);
 }
 
+rocksdb::Status Storage::Get(const rocksdb::ReadOptions &options, const rocksdb::Slice &key,
+                             rocksdb::PinnableSlice *value) {
+  return Get(options, db_->DefaultColumnFamily(), key, value);
+}
+
+rocksdb::Status Storage::Get(const rocksdb::ReadOptions &options, rocksdb::ColumnFamilyHandle *column_family,
+                             const rocksdb::Slice &key, rocksdb::PinnableSlice *value) {
+  if (is_txn_mode_ && txn_write_batch_->GetWriteBatch()->Count() > 0) {
+    return txn_write_batch_->GetFromBatchAndDB(db_.get(), options, column_family, key, value);
+  }
+  return db_->Get(options, column_family, key, value);
+}
+
 rocksdb::Iterator *Storage::NewIterator(const rocksdb::ReadOptions &options) {
   return NewIterator(options, db_->DefaultColumnFamily());
 }
@@ -597,7 +613,7 @@ rocksdb::Status Storage::DeleteRange(const std::string &first_key, const std::st
 }
 
 rocksdb::Status Storage::FlushScripts(const rocksdb::WriteOptions &options, rocksdb::ColumnFamilyHandle *cf_handle) {
-  std::string begin_key = kLuaFunctionPrefix, end_key = begin_key;
+  std::string begin_key = kLuaFuncSHAPrefix, end_key = begin_key;
   // we need to increase one here since the DeleteRange api
   // didn't contain the end key.
   end_key[end_key.size() - 1] += 1;
@@ -640,13 +656,14 @@ rocksdb::ColumnFamilyHandle *Storage::GetCFHandle(const std::string &name) {
   return cf_handles_[0];
 }
 
-rocksdb::Status Storage::Compact(const Slice *begin, const Slice *end) {
+rocksdb::Status Storage::Compact(rocksdb::ColumnFamilyHandle *cf, const Slice *begin, const Slice *end) {
   rocksdb::CompactRangeOptions compact_opts;
   compact_opts.change_level = true;
   // For the manual compaction, we would like to force the bottommost level to be compacted.
   // Or it may use the trivial mode and some expired key-values were still exist in the bottommost level.
   compact_opts.bottommost_level_compaction = rocksdb::BottommostLevelCompaction::kForceOptimized;
-  for (const auto &cf_handle : cf_handles_) {
+  const auto &cf_handles = cf ? std::vector<rocksdb::ColumnFamilyHandle *>{cf} : cf_handles_;
+  for (const auto &cf_handle : cf_handles) {
     rocksdb::Status s = db_->CompactRange(compact_opts, cf_handle, begin, end);
     if (!s.ok()) return s;
   }
@@ -755,20 +772,20 @@ Status Storage::WriteToPropagateCF(const std::string &key, const std::string &va
 }
 
 Status Storage::ShiftReplId() {
-  const char *charset = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
-  const int charset_len = static_cast<int>(strlen(charset));
+  static constexpr std::string_view charset = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
   // Do nothing if rsid psync is not enabled
   if (!config_->use_rsid_psync) return Status::OK();
 
   std::random_device rd;
   std::mt19937 gen(rd() + getpid());
-  std::uniform_int_distribution<> distrib(0, charset_len - 1);
-  std::string rand_str;
+  std::uniform_int_distribution<size_t> distrib(0, charset.size() - 1);
+
+  std::string rand_str(kReplIdLength, 0);
   for (int i = 0; i < kReplIdLength; i++) {
-    rand_str.push_back(charset[distrib(gen)]);
+    rand_str[i] = charset[distrib(gen)];
   }
-  replid_ = rand_str;
+  replid_ = std::move(rand_str);
   LOG(INFO) << "[replication] New replication id: " << replid_;
 
   // Write new replication id into db engine
