@@ -27,10 +27,11 @@
 #include <vector>
 
 #include "db_util.h"
+#include "time_util.h"
 
 namespace redis {
 
-constexpr const char *consumerGroupMetadataDelimiter = "METADATA";
+std::string_view consumerGroupMetadataDelimiter = "METADATA";
 const char *errSetEntryIdSmallerThanLastGenerated =
     "The ID specified in XSETID is smaller than the target stream top item";
 const char *errEntriesAddedSmallerThanStreamSize =
@@ -180,9 +181,7 @@ std::string Stream::groupNameFromInternalKey(rocksdb::Slice key) const {
   uint64_t len = 0;
   GetFixed64(&group_name_metadata, &len);
   std::string group_name;
-  if (len <= group_name_metadata.size() - strlen(consumerGroupMetadataDelimiter)) {
-    group_name = group_name_metadata.ToString().substr(0, len);
-  }
+  group_name = group_name_metadata.ToString().substr(0, len);
   return group_name;
 }
 
@@ -211,6 +210,66 @@ StreamConsumerGroupMetadata Stream::decodeStreamConsumerGroupMetadataValue(const
   return consumer_group_metadata;
 }
 
+std::string Stream::internalKeyFromConsumerName(const std::string &ns_key, const StreamMetadata &metadata,
+                                                const std::string &group_name, const std::string &consumer_name) const {
+  std::string sub_key;
+  PutFixed64(&sub_key, group_name.size());
+  sub_key += group_name;
+  PutFixed64(&sub_key, consumer_name.size());
+  sub_key += consumer_name;
+  sub_key += consumerGroupMetadataDelimiter;
+  std::string entry_key = InternalKey(ns_key, sub_key, metadata.version, storage_->IsSlotIdEncoded()).Encode();
+  return entry_key;
+}
+
+std::string Stream::consumerNameFromInternalKey(rocksdb::Slice key) const {
+  InternalKey ikey(key, storage_->IsSlotIdEncoded());
+  Slice subkey = ikey.GetSubKey();
+  uint64_t group_name_len = 0;
+  GetFixed64(&subkey, &group_name_len);
+  subkey.remove_prefix(group_name_len);
+  uint64_t consumer_name_len = 0;
+  GetFixed64(&subkey, &consumer_name_len);
+  return subkey.ToString().substr(0, consumer_name_len);
+}
+
+std::string Stream::encodeStreamConsumerMetadataValue(const StreamConsumerMetadata &consumer_metadata) {
+  std::string dst;
+  PutFixed64(&dst, consumer_metadata.pending_number);
+  PutFixed64(&dst, consumer_metadata.last_idle);
+  PutFixed64(&dst, consumer_metadata.last_active);
+  return dst;
+}
+
+StreamConsumerMetadata Stream::decodeStreamConsumerMetadataValue(const std::string &value) {
+  StreamConsumerMetadata consumer_metadata;
+  rocksdb::Slice input(value);
+  GetFixed64(&input, &consumer_metadata.pending_number);
+  GetFixed64(&input, &consumer_metadata.last_idle);
+  GetFixed64(&input, &consumer_metadata.last_active);
+  return consumer_metadata;
+}
+
+StreamSubkeyType Stream::identifySubkeyType(const rocksdb::Slice &key) {
+  InternalKey ikey(key, storage_->IsSlotIdEncoded());
+  Slice subkey = ikey.GetSubKey();
+  const size_t entry_id_size = sizeof(StreamEntryID);
+  if (subkey.size() <= entry_id_size) {
+    return StreamSubkeyType::StreamEntry;
+  }
+  uint64_t group_name_len = 0;
+  GetFixed64(&subkey, &group_name_len);
+  std::string without_group_name = subkey.ToString().substr(group_name_len);
+  const size_t metadata_delimiter_size = consumerGroupMetadataDelimiter.size();
+  if (without_group_name.size() <= metadata_delimiter_size) {
+    return StreamSubkeyType::StreamConsumerGroupMetadata;
+  }
+  if (without_group_name.size() <= entry_id_size) {
+    return StreamSubkeyType::StreamPelEntry;
+  }
+  return StreamSubkeyType::StreamConsumerMetadata;
+}
+
 rocksdb::Status Stream::CreateGroup(const Slice &stream_name, const StreamXGroupCreateOptions &options,
                                     const std::string &group_name) {
   if (std::isdigit(group_name[0])) {
@@ -313,6 +372,102 @@ rocksdb::Status Stream::DestroyGroup(const Slice &stream_name, const std::string
   return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
+rocksdb::Status Stream::CreateConsumer(const Slice &stream_name, const std::string &group_name,
+                                       const std::string &consumer_name, int *created_number) {
+  if (std::isdigit(consumer_name[0])) {
+    return rocksdb::Status::InvalidArgument("consumer name cannot start with number");
+  }
+  std::string ns_key = AppendNamespacePrefix(stream_name);
+  LockGuard guard(storage_->GetLockManager(), ns_key);
+  StreamMetadata metadata;
+  rocksdb::Status s = GetMetadata(ns_key, &metadata);
+  if (!s.ok() && !s.IsNotFound()) {
+    return s;
+  }
+  if (s.IsNotFound()) {
+    return rocksdb::Status::InvalidArgument(errXGroupSubcommandRequiresKeyExist);
+  }
+
+  std::string entry_key = internalKeyFromGroupName(ns_key, metadata, group_name);
+  std::string get_entry_value;
+  s = storage_->Get(rocksdb::ReadOptions(), stream_cf_handle_, entry_key, &get_entry_value);
+  if (!s.ok() && !s.IsNotFound()) {
+    return s;
+  }
+  if (s.IsNotFound()) {
+    return rocksdb::Status::InvalidArgument("NOGROUP No such consumer group " + group_name + " for key name " +
+                                            stream_name.ToString());
+  }
+
+  StreamConsumerMetadata consumer_metadata;
+  auto now = util::GetTimeStampMS();
+  consumer_metadata.last_idle = now;
+  consumer_metadata.last_active = now;
+  std::string consumer_key = internalKeyFromConsumerName(ns_key, metadata, group_name, consumer_name);
+  std::string consumer_value = encodeStreamConsumerMetadataValue(consumer_metadata);
+  std::string get_consumer_value;
+  s = storage_->Get(rocksdb::ReadOptions(), stream_cf_handle_, consumer_key, &get_consumer_value);
+  if (!s.IsNotFound()) {
+    return s;
+  }
+
+  auto batch = storage_->GetWriteBatchBase();
+  WriteBatchLogData log_data(kRedisStream);
+  batch->PutLogData(log_data.Encode());
+
+  batch->Put(stream_cf_handle_, consumer_key, consumer_value);
+  StreamConsumerGroupMetadata consumer_group_metadata = decodeStreamConsumerGroupMetadataValue(get_entry_value);
+  consumer_group_metadata.consumer_number += 1;
+  std::string consumer_group_metadata_bytes = encodeStreamConsumerGroupMetadataValue(consumer_group_metadata);
+  batch->Put(stream_cf_handle_, entry_key, consumer_group_metadata_bytes);
+  s = storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  if (s.ok()) *created_number = 1;
+  return s;
+}
+
+rocksdb::Status Stream::GroupSetId(const Slice &stream_name, const std::string &group_name,
+                                   const StreamXGroupCreateOptions &options) {
+  std::string ns_key = AppendNamespacePrefix(stream_name);
+  LockGuard guard(storage_->GetLockManager(), ns_key);
+  StreamMetadata metadata;
+  rocksdb::Status s = GetMetadata(ns_key, &metadata);
+  if (!s.ok() && !s.IsNotFound()) {
+    return s;
+  }
+  if (s.IsNotFound()) {
+    return rocksdb::Status::InvalidArgument(errXGroupSubcommandRequiresKeyExist);
+  }
+
+  std::string entry_key = internalKeyFromGroupName(ns_key, metadata, group_name);
+  std::string get_entry_value;
+  s = storage_->Get(rocksdb::ReadOptions(), stream_cf_handle_, entry_key, &get_entry_value);
+  if (!s.ok() && !s.IsNotFound()) {
+    return s;
+  }
+  if (s.IsNotFound()) {
+    return rocksdb::Status::InvalidArgument("NOGROUP No such consumer group " + group_name + " for key name " +
+                                            stream_name.ToString());
+  }
+
+  StreamConsumerGroupMetadata consumer_group_metadata = decodeStreamConsumerGroupMetadataValue(get_entry_value);
+  if (options.last_id == "$") {
+    consumer_group_metadata.last_delivered_id = metadata.last_entry_id;
+  } else {
+    auto s = ParseStreamEntryID(options.last_id, &consumer_group_metadata.last_delivered_id);
+    if (!s.IsOK()) {
+      return rocksdb::Status::InvalidArgument(s.Msg());
+    }
+  }
+  consumer_group_metadata.entries_read = options.entries_read;
+  std::string entry_value = encodeStreamConsumerGroupMetadataValue(consumer_group_metadata);
+
+  auto batch = storage_->GetWriteBatchBase();
+  WriteBatchLogData log_data(kRedisStream);
+  batch->PutLogData(log_data.Encode());
+  batch->Put(stream_cf_handle_, entry_key, entry_value);
+  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+}
+
 rocksdb::Status Stream::DeleteEntries(const Slice &stream_name, const std::vector<StreamEntryID> &ids,
                                       uint64_t *deleted_cnt) {
   *deleted_cnt = 0;
@@ -620,6 +775,126 @@ rocksdb::Status Stream::GetStreamInfo(const rocksdb::Slice &stream_name, bool fu
   return rocksdb::Status::OK();
 }
 
+static bool StreamRangeHasTombstones(const StreamMetadata &metadata, StreamEntryID start_id) {
+  StreamEntryID end_id = StreamEntryID{UINT64_MAX, UINT64_MAX};
+  if (metadata.size == 0 || metadata.max_deleted_entry_id == StreamEntryID{0, 0}) {
+    return false;
+  }
+  if (metadata.first_entry_id > metadata.max_deleted_entry_id) {
+    return false;
+  }
+  return (start_id <= metadata.max_deleted_entry_id && metadata.max_deleted_entry_id <= end_id);
+}
+
+static int64_t StreamEstimateDistanceFromFirstEverEntry(const StreamMetadata &metadata, StreamEntryID id) {
+  if (metadata.entries_added == 0) {
+    return 0;
+  }
+  if (metadata.size == 0 && id < metadata.last_entry_id) {
+    return static_cast<int64_t>(metadata.entries_added);
+  }
+  if (id == metadata.last_entry_id) {
+    return static_cast<int64_t>(metadata.entries_added);
+  } else if (id > metadata.last_entry_id) {
+    return -1;
+  }
+  if (metadata.max_deleted_entry_id == StreamEntryID{0, 0} || metadata.max_deleted_entry_id < metadata.first_entry_id) {
+    if (id < metadata.first_entry_id) {
+      return static_cast<int64_t>(metadata.entries_added - metadata.size);
+    } else if (id == metadata.first_entry_id) {
+      return static_cast<int64_t>(metadata.entries_added - metadata.size + 1);
+    }
+  }
+  return -1;
+}
+
+static void CheckLagValid(const StreamMetadata &stream_metadata, StreamConsumerGroupMetadata &group_metadata) {
+  bool valid = false;
+  if (stream_metadata.entries_added == 0) {
+    group_metadata.lag = 0;
+    valid = true;
+  } else if (group_metadata.entries_read != -1 &&
+             !StreamRangeHasTombstones(stream_metadata, group_metadata.last_delivered_id)) {
+    group_metadata.lag = stream_metadata.entries_added - group_metadata.entries_read;
+    valid = true;
+  } else {
+    int64_t entries_read = StreamEstimateDistanceFromFirstEverEntry(stream_metadata, group_metadata.last_delivered_id);
+    if (entries_read != -1) {
+      group_metadata.lag = stream_metadata.entries_added - entries_read;
+      valid = true;
+    }
+  }
+  if (!valid) {
+    group_metadata.lag = UINT64_MAX;
+  }
+}
+
+rocksdb::Status Stream::GetGroupInfo(const Slice &stream_name,
+                                     std::vector<std::pair<std::string, StreamConsumerGroupMetadata>> &group_metadata) {
+  std::string ns_key = AppendNamespacePrefix(stream_name);
+  StreamMetadata metadata(false);
+  rocksdb::Status s = GetMetadata(ns_key, &metadata);
+  if (!s.ok()) return s;
+
+  std::string next_version_prefix_key =
+      InternalKey(ns_key, "", metadata.version + 1, storage_->IsSlotIdEncoded()).Encode();
+  std::string prefix_key = InternalKey(ns_key, "", metadata.version, storage_->IsSlotIdEncoded()).Encode();
+
+  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
+  LatestSnapShot ss(storage_);
+  read_options.snapshot = ss.GetSnapShot();
+  rocksdb::Slice upper_bound(next_version_prefix_key);
+  read_options.iterate_upper_bound = &upper_bound;
+  rocksdb::Slice lower_bound(prefix_key);
+  read_options.iterate_lower_bound = &lower_bound;
+
+  auto iter = util::UniqueIterator(storage_, read_options, stream_cf_handle_);
+  for (iter->SeekToFirst(); iter->Valid(); iter->Next()) {
+    if (identifySubkeyType(iter->key()) == StreamSubkeyType::StreamConsumerGroupMetadata) {
+      std::string group_name = groupNameFromInternalKey(iter->key());
+      StreamConsumerGroupMetadata cg_metadata = decodeStreamConsumerGroupMetadataValue(iter->value().ToString());
+      CheckLagValid(metadata, cg_metadata);
+      std::pair<std::string, StreamConsumerGroupMetadata> tmp_item(group_name, cg_metadata);
+      group_metadata.push_back(tmp_item);
+    }
+  }
+  return rocksdb::Status::OK();
+}
+
+rocksdb::Status Stream::GetConsumerInfo(
+    const Slice &stream_name, const std::string &group_name,
+    std::vector<std::pair<std::string, StreamConsumerMetadata>> &consumer_metadata) {
+  std::string ns_key = AppendNamespacePrefix(stream_name);
+  StreamMetadata metadata(false);
+  rocksdb::Status s = GetMetadata(ns_key, &metadata);
+  if (!s.ok()) return s;
+
+  std::string next_version_prefix_key =
+      InternalKey(ns_key, "", metadata.version + 1, storage_->IsSlotIdEncoded()).Encode();
+  std::string prefix_key = InternalKey(ns_key, "", metadata.version, storage_->IsSlotIdEncoded()).Encode();
+
+  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
+  LatestSnapShot ss(storage_);
+  read_options.snapshot = ss.GetSnapShot();
+  rocksdb::Slice upper_bound(next_version_prefix_key);
+  read_options.iterate_upper_bound = &upper_bound;
+  rocksdb::Slice lower_bound(prefix_key);
+  read_options.iterate_lower_bound = &lower_bound;
+
+  auto iter = util::UniqueIterator(storage_, read_options, stream_cf_handle_);
+  for (iter->SeekToFirst(); iter->Valid(); iter->Next()) {
+    if (identifySubkeyType(iter->key()) == StreamSubkeyType::StreamConsumerMetadata) {
+      std::string cur_group_name = groupNameFromInternalKey(iter->key());
+      if (cur_group_name != group_name) continue;
+      std::string consumer_name = consumerNameFromInternalKey(iter->key());
+      StreamConsumerMetadata c_metadata = decodeStreamConsumerMetadataValue(iter->value().ToString());
+      std::pair<std::string, StreamConsumerMetadata> tmp_item(consumer_name, c_metadata);
+      consumer_metadata.push_back(tmp_item);
+    }
+  }
+  return rocksdb::Status::OK();
+}
+
 rocksdb::Status Stream::Range(const Slice &stream_name, const StreamRangeOptions &options,
                               std::vector<StreamEntry> *entries) {
   entries->clear();
