@@ -42,9 +42,17 @@ class Stream : public SubKeyScanner {
   rocksdb::Status CreateGroup(const Slice &stream_name, const StreamXGroupCreateOptions &options,
                               const std::string &group_name);
   rocksdb::Status DestroyGroup(const Slice &stream_name, const std::string &group_name, uint64_t *delete_cnt);
+  rocksdb::Status CreateConsumer(const Slice &stream_name, const std::string &group_name,
+                                 const std::string &consumer_name, int *created_number);
+  rocksdb::Status GroupSetId(const Slice &stream_name, const std::string &group_name,
+                             const StreamXGroupCreateOptions &options);
   rocksdb::Status DeleteEntries(const Slice &stream_name, const std::vector<StreamEntryID> &ids, uint64_t *deleted_cnt);
   rocksdb::Status Len(const Slice &stream_name, const StreamLenOptions &options, uint64_t *size);
   rocksdb::Status GetStreamInfo(const Slice &stream_name, bool full, uint64_t count, StreamInfo *info);
+  rocksdb::Status GetGroupInfo(const Slice &stream_name,
+                               std::vector<std::pair<std::string, StreamConsumerGroupMetadata>> &group_metadata);
+  rocksdb::Status GetConsumerInfo(const Slice &stream_name, const std::string &group_name,
+                                  std::vector<std::pair<std::string, StreamConsumerMetadata>> &consumer_metadata);
   rocksdb::Status Range(const Slice &stream_name, const StreamRangeOptions &options, std::vector<StreamEntry> *entries);
   rocksdb::Status Trim(const Slice &stream_name, const StreamTrimOptions &options, uint64_t *delete_cnt);
   rocksdb::Status GetMetadata(const Slice &stream_name, StreamMetadata *metadata);
@@ -69,6 +77,12 @@ class Stream : public SubKeyScanner {
   std::string groupNameFromInternalKey(rocksdb::Slice key) const;
   static std::string encodeStreamConsumerGroupMetadataValue(const StreamConsumerGroupMetadata &consumer_group_metadata);
   static StreamConsumerGroupMetadata decodeStreamConsumerGroupMetadataValue(const std::string &value);
+  std::string internalKeyFromConsumerName(const std::string &ns_key, const StreamMetadata &metadata,
+                                          const std::string &group_name, const std::string &consumer_name) const;
+  std::string consumerNameFromInternalKey(rocksdb::Slice key) const;
+  static std::string encodeStreamConsumerMetadataValue(const StreamConsumerMetadata &consumer_metadata);
+  static StreamConsumerMetadata decodeStreamConsumerMetadataValue(const std::string &value);
+  StreamSubkeyType identifySubkeyType(const rocksdb::Slice &key);
 };
 
 }  // namespace redis
