@@ -31,14 +31,20 @@ class RedisListTest : public TestBase {
   ~RedisListTest() override = default;
 
   void SetUp() override {
+    // Assume that `field_` is a matrix of size `m_` * `n_`,
+    // where every row is identical, and each element within a single row is distinct.
     key_ = "test-list-key";
     fields_ = {"list-test-key-1", "list-test-key-2", "list-test-key-3", "list-test-key-4", "list-test-key-5",
                "list-test-key-1", "list-test-key-2", "list-test-key-3", "list-test-key-4", "list-test-key-5",
                "list-test-key-1", "list-test-key-2", "list-test-key-3", "list-test-key-4", "list-test-key-5",
                "list-test-key-1", "list-test-key-2", "list-test-key-3", "list-test-key-4", "list-test-key-5"};
+    m_ = 4;
+    n_ = 5;
   }
 
   std::unique_ptr<redis::List> list_;
+  int n_;
+  int m_;
 };
 
 class RedisListSpecificTest : public RedisListTest {
@@ -52,15 +58,15 @@ class RedisListSpecificTest : public RedisListTest {
 class RedisListLMoveTest : public RedisListTest {
  protected:
   void SetUp() override {
-    list_->Del(key_);
-    list_->Del(dst_key_);
+    auto s = list_->Del(key_);
+    s = list_->Del(dst_key_);
     fields_ = {"src1", "src2", "src3", "src4"};
     dst_fields_ = {"dst", "dst2", "dst3", "dst4"};
   }
 
   void TearDown() override {
-    list_->Del(key_);
-    list_->Del(dst_key_);
+    auto s = list_->Del(key_);
+    s = list_->Del(dst_key_);
   }
 
   void listElementsAreEqualTo(const Slice &key, int start, int stop, const std::vector<Slice> &expected_elems) {
@@ -95,7 +101,7 @@ TEST_F(RedisListTest, PushAndPop) {
     list_->Pop(key_, true, &elem);
     EXPECT_EQ(elem, field.ToString());
   }
-  list_->Del(key_);
+  auto s = list_->Del(key_);
 }
 
 TEST_F(RedisListTest, Pushx) {
@@ -108,7 +114,7 @@ TEST_F(RedisListTest, Pushx) {
   s = list_->PushX(pushx_key, fields_, true, &ret);
   EXPECT_TRUE(s.ok());
   EXPECT_EQ(ret, fields_.size() * 2);
-  list_->Del(pushx_key);
+  s = list_->Del(pushx_key);
 }
 
 TEST_F(RedisListTest, Index) {
@@ -126,7 +132,7 @@ TEST_F(RedisListTest, Index) {
   }
   rocksdb::Status s = list_->Index(key_, -1, &elem);
   EXPECT_TRUE(s.IsNotFound());
-  list_->Del(key_);
+  s = list_->Del(key_);
 }
 
 TEST_F(RedisListTest, Set) {
@@ -141,7 +147,7 @@ TEST_F(RedisListTest, Set) {
   for (size_t i = 0; i < fields_.size(); i++) {
     list_->Pop(key_, true, &elem);
   }
-  list_->Del(key_);
+  auto s = list_->Del(key_);
 }
 
 TEST_F(RedisListTest, Range) {
@@ -159,7 +165,90 @@ TEST_F(RedisListTest, Range) {
     list_->Pop(key_, true, &elem);
     EXPECT_EQ(elem, field.ToString());
   }
-  list_->Del(key_);
+  auto s = list_->Del(key_);
+}
+
+TEST_F(RedisListTest, Pos) {
+  uint64_t ret = 0;
+  list_->Push(key_, fields_, false, &ret);
+  EXPECT_EQ(fields_.size(), ret);
+
+  // Basic usage
+  PosSpec spec;
+  std::vector<int64_t> indexes;
+  list_->Pos(key_, fields_[0], spec, &indexes);
+  EXPECT_EQ(1, indexes.size());
+  EXPECT_EQ(0, indexes[0]);
+  list_->Pos(key_, fields_[2], spec, &indexes);
+  EXPECT_EQ(1, indexes.size());
+  EXPECT_EQ(2, indexes[0]);
+
+  // RANK option
+  spec = PosSpec{};
+  spec.rank = m_ + 1;
+  auto s = list_->Pos(key_, fields_[3], spec, &indexes);
+  EXPECT_TRUE(s.ok());
+  EXPECT_TRUE(indexes.empty());
+  spec.rank = -(m_ + 1);
+  s = list_->Pos(key_, fields_[3], spec, &indexes);
+  EXPECT_TRUE(s.ok());
+  EXPECT_TRUE(indexes.empty());
+  // positive
+  for (int i = 1; i <= m_; ++i) {
+    spec.rank = i;
+    list_->Pos(key_, fields_[3], spec, &indexes);
+    EXPECT_EQ(1, indexes.size());
+    EXPECT_EQ(n_ * (i - 1) + 3, indexes[0]);
+  }
+  // negative
+  for (int i = 1; i <= m_; ++i) {
+    spec.rank = -i;
+    list_->Pos(key_, fields_[3], spec, &indexes);
+    EXPECT_EQ(1, indexes.size());
+    EXPECT_EQ(fields_.size() - n_ * i + 3, indexes[0]);
+  }
+
+  // COUNT option
+  spec = PosSpec{};
+  spec.count = 0;
+  list_->Pos(key_, fields_[3], spec, &indexes);
+  EXPECT_EQ(m_, indexes.size());
+  EXPECT_EQ(3, indexes[0]);
+  spec.count = 2;
+  list_->Pos(key_, fields_[3], spec, &indexes);
+  EXPECT_EQ(2, indexes.size());
+  EXPECT_EQ(3, indexes[0]);
+  spec.count = 100;
+  list_->Pos(key_, fields_[3], spec, &indexes);
+  EXPECT_EQ(m_, indexes.size());
+  EXPECT_EQ(3, indexes[0]);
+
+  // COUNT + RANK option
+  spec = PosSpec{};
+  spec.count = 0;
+  spec.rank = 2;
+  list_->Pos(key_, fields_[3], spec, &indexes);
+  EXPECT_EQ(n_ - 2, indexes.size());
+  spec.count = 2;
+  spec.rank = -2;
+  list_->Pos(key_, fields_[3], spec, &indexes);
+  EXPECT_EQ(2, indexes.size());
+
+  // MAXLEN option
+  PosSpec maxlen_spec;
+  maxlen_spec.max_len = 2;
+  list_->Pos(key_, fields_[2], maxlen_spec, &indexes);
+  EXPECT_TRUE(indexes.empty());
+  list_->Pos(key_, fields_[1], maxlen_spec, &indexes);
+  EXPECT_EQ(1, indexes.size());
+  EXPECT_EQ(1, indexes[0]);
+  maxlen_spec.count = 0;
+  maxlen_spec.max_len = (n_ * 2);
+  list_->Pos(key_, fields_[3], maxlen_spec, &indexes);
+  EXPECT_EQ(2, indexes.size());
+  EXPECT_EQ(3, indexes[0]);
+
+  s = list_->Del(key_);
 }
 
 TEST_F(RedisListTest, Rem) {
@@ -232,7 +321,7 @@ TEST_F(RedisListTest, Rem) {
     list_->Pop(key_, true, &elem);
     EXPECT_EQ(elem, field.ToString());
   }
-  list_->Del(key_);
+  auto s = list_->Del(key_);
 }
 
 TEST_F(RedisListSpecificTest, Rem) {
@@ -270,7 +359,7 @@ TEST_F(RedisListSpecificTest, Rem) {
     list_->Pop(key_, false, &elem);
     EXPECT_EQ(elem, fields_[i - 1].ToString());
   }
-  list_->Del(key_);
+  auto s = list_->Del(key_);
 }
 
 TEST_F(RedisListTest, Trim) {
@@ -286,7 +375,7 @@ TEST_F(RedisListTest, Trim) {
     list_->Pop(key_, true, &elem);
     EXPECT_EQ(elem, fields_[i].ToString());
   }
-  list_->Del(key_);
+  auto s = list_->Del(key_);
 }
 
 TEST_F(RedisListSpecificTest, Trim) {
@@ -311,26 +400,7 @@ TEST_F(RedisListSpecificTest, Trim) {
     list_->Pop(key_, true, &elem);
     EXPECT_EQ(elem, fields_[i].ToString());
   }
-  list_->Del(key_);
-}
-
-TEST_F(RedisListTest, RPopLPush) {
-  uint64_t ret = 0;
-  list_->Push(key_, fields_, true, &ret);
-  EXPECT_EQ(fields_.size(), ret);
-  Slice dst("test-list-rpoplpush-key");
-  for (auto &field : fields_) {
-    std::string elem;
-    list_->RPopLPush(key_, dst, &elem);
-    EXPECT_EQ(field.ToString(), elem);
-  }
-  for (auto &field : fields_) {
-    std::string elem;
-    list_->Pop(dst, false, &elem);
-    EXPECT_EQ(elem, field.ToString());
-  }
-  list_->Del(key_);
-  list_->Del(dst);
+  auto s = list_->Del(key_);
 }
 
 TEST_F(RedisListLMoveTest, LMoveSrcNotExist) {
@@ -451,9 +521,9 @@ TEST_F(RedisListLMoveTest, LMoveSrcRightDstRight) {
 
 TEST_F(RedisListTest, LPopEmptyList) {
   std::string non_existing_key{"non-existing-key"};
-  list_->Del(non_existing_key);
+  auto s = list_->Del(non_existing_key);
   std::string elem;
-  auto s = list_->Pop(non_existing_key, true, &elem);
+  s = list_->Pop(non_existing_key, true, &elem);
   EXPECT_TRUE(s.IsNotFound());
   std::vector<std::string> elems;
   s = list_->PopMulti(non_existing_key, true, 10, &elems);
@@ -472,7 +542,7 @@ TEST_F(RedisListTest, LPopOneElement) {
   std::string elem;
   auto s = list_->Pop(key_, true, &elem);
   EXPECT_TRUE(s.IsNotFound());
-  list_->Del(key_);
+  s = list_->Del(key_);
 }
 
 TEST_F(RedisListTest, LPopMulti) {
@@ -487,7 +557,7 @@ TEST_F(RedisListTest, LPopMulti) {
   for (size_t i = 0; i < elems.size(); ++i) {
     EXPECT_EQ(elems[i], fields_[i].ToString());
   }
-  list_->Del(key_);
+  s = list_->Del(key_);
 }
 
 TEST_F(RedisListTest, LPopMultiCountGreaterThanListSize) {
@@ -501,14 +571,14 @@ TEST_F(RedisListTest, LPopMultiCountGreaterThanListSize) {
   for (size_t i = 0; i < elems.size(); ++i) {
     EXPECT_EQ(elems[i], fields_[i].ToString());
   }
-  list_->Del(key_);
+  s = list_->Del(key_);
 }
 
 TEST_F(RedisListTest, RPopEmptyList) {
   std::string non_existing_key{"non-existing-key"};
-  list_->Del(non_existing_key);
+  auto s = list_->Del(non_existing_key);
   std::string elem;
-  auto s = list_->Pop(non_existing_key, false, &elem);
+  s = list_->Pop(non_existing_key, false, &elem);
   EXPECT_TRUE(s.IsNotFound());
   std::vector<std::string> elems;
   s = list_->PopMulti(non_existing_key, false, 10, &elems);
@@ -527,7 +597,7 @@ TEST_F(RedisListTest, RPopOneElement) {
   std::string elem;
   auto s = list_->Pop(key_, false, &elem);
   EXPECT_TRUE(s.IsNotFound());
-  list_->Del(key_);
+  s = list_->Del(key_);
 }
 
 TEST_F(RedisListTest, RPopMulti) {
@@ -542,7 +612,7 @@ TEST_F(RedisListTest, RPopMulti) {
   for (size_t i = 0; i < elems.size(); ++i) {
     EXPECT_EQ(elems[i], fields_[fields_.size() - i - 1].ToString());
   }
-  list_->Del(key_);
+  s = list_->Del(key_);
 }
 
 TEST_F(RedisListTest, RPopMultiCountGreaterThanListSize) {
@@ -556,5 +626,5 @@ TEST_F(RedisListTest, RPopMultiCountGreaterThanListSize) {
   for (size_t i = 0; i < elems.size(); ++i) {
     EXPECT_EQ(elems[i], fields_[fields_.size() - i - 1].ToString());
   }
-  list_->Del(key_);
+  s = list_->Del(key_);
 }
