@@ -110,7 +110,7 @@ def build(dir: str, jobs: Optional[int], ghproxy: bool, ninja: bool, unittest: b
 
     cmake_options = ["-DCMAKE_BUILD_TYPE=RelWithDebInfo"]
     if ghproxy:
-        cmake_options.append("-DDEPS_FETCH_PROXY=https://ghproxy.com/")
+        cmake_options.append("-DDEPS_FETCH_PROXY=https://mirror.ghproxy.com/")
     if ninja:
         cmake_options.append("-G Ninja")
     if compiler == 'gcc':
@@ -356,7 +356,7 @@ if __name__ == '__main__':
                               help="directory to store cmake-generated and build files")
     parser_build.add_argument('-j', '--jobs', metavar='N', help='execute N build jobs concurrently')
     parser_build.add_argument('--ghproxy', default=False, action='store_true',
-                              help='use https://ghproxy.com to fetch dependencies')
+                              help='use https://mirror.ghproxy.com to fetch dependencies')
     parser_build.add_argument('--ninja', default=False, action='store_true', help='use Ninja to build kvrocks')
     parser_build.add_argument('--unittest', default=False, action='store_true', help='build unittest target')
     parser_build.add_argument('--compiler', default='auto', choices=('auto', 'gcc', 'clang'),
